@@ -17,23 +17,25 @@ struct t_protocolInfo
 };
 
 static const t_protocolInfo protocolInfos[] = {
-	{ FTP,          L"ftp",    false, 21,  true,  fztranslate_mark("FTP - File Transfer Protocol with optional encryption"), L"" },
-	{ SFTP,         L"sftp",   true,  22,  false, "SFTP - SSH File Transfer Protocol",                                       L"" },
-	{ HTTP,         L"http",   true,  80,  false, "HTTP - Hypertext Transfer Protocol",                                      L"" },
-	{ HTTPS,        L"https",  true, 443,  true,  fztranslate_mark("HTTPS - HTTP over TLS"),                                 L"" },
-	{ FTPS,         L"ftps",   true, 990,  true,  fztranslate_mark("FTPS - FTP over implicit TLS"),                          L"" },
-	{ FTPES,        L"ftpes",  true,  21,  true,  fztranslate_mark("FTPES - FTP over explicit TLS"),                         L"" },
-	{ INSECURE_FTP, L"ftp",    false, 21,  true,  fztranslate_mark("FTP - Insecure File Transfer Protocol"),                 L"" },
-	{ S3,           L"s3",     true, 443,  false, "S3 - Amazon Simple Storage Service",                                      L"" },
-	{ STORJ,        L"storj",  true, 443,  true,  fztranslate_mark("Storj - Decentralized Cloud Storage"),                   L"" },
-	{ WEBDAV,       L"webdav", true, 443,  true,  "WebDAV",                                                                  L"https" },
-	{ AZURE_FILE,   L"azfile", true, 443,  false, "Microsoft Azure File Storage Service",                                    L"https" },
-	{ AZURE_BLOB,   L"azblob", true, 443,  false, "Microsoft Azure Blob Storage Service",                                    L"https" },
-	{ SWIFT,        L"swift",  true, 443,  false, "OpenStack Swift",                                                         L"https" },
-	{ GOOGLE_CLOUD, L"google", true, 443,  false, "Google Cloud Storage",                                                    L"https" },
-	{ GOOGLE_DRIVE, L"gdrive", true, 443,  false, "Google Drive",                                                            L"https" },
-	{ DROPBOX,		L"dropbox", true, 443, false, "Dropbox",                                                                 L"https" },
-	{ UNKNOWN,      L"",       false, 21,  false, "", L"" }
+	{ FTP,          L"ftp",      false, 21, true,  fztranslate_mark("FTP - File Transfer Protocol with optional encryption"), L"" },
+	{ SFTP,         L"sftp",     true,  22, false, "SFTP - SSH File Transfer Protocol",                                       L"" },
+	{ HTTP,         L"http",     true,  80, false, "HTTP - Hypertext Transfer Protocol",                                      L"" },
+	{ HTTPS,        L"https",    true, 443, true,  fztranslate_mark("HTTPS - HTTP over TLS"),                                 L"" },
+	{ FTPS,         L"ftps",     true, 990, true,  fztranslate_mark("FTPS - FTP over implicit TLS"),                          L"" },
+	{ FTPES,        L"ftpes",    true,  21, true,  fztranslate_mark("FTPES - FTP over explicit TLS"),                         L"" },
+	{ INSECURE_FTP, L"ftp",      false, 21, true,  fztranslate_mark("FTP - Insecure File Transfer Protocol"),                 L"" },
+	{ S3,           L"s3",       true, 443, false, "S3 - Amazon Simple Storage Service",                                      L"" },
+	{ STORJ,        L"storj",    true, 443, true,  fztranslate_mark("Storj - Decentralized Cloud Storage"),                   L"" },
+	{ WEBDAV,       L"webdav",   true, 443, true,  "WebDAV",                                                                  L"https" },
+	{ AZURE_FILE,   L"azfile",   true, 443, false, "Microsoft Azure File Storage Service",                                    L"https" },
+	{ AZURE_BLOB,   L"azblob",   true, 443, false, "Microsoft Azure Blob Storage Service",                                    L"https" },
+	{ SWIFT,        L"swift",    true, 443, false, "OpenStack Swift",                                                         L"https" },
+	{ GOOGLE_CLOUD, L"google",   true, 443, false, "Google Cloud Storage",                                                    L"https" },
+	{ GOOGLE_DRIVE, L"gdrive",   true, 443, false, "Google Drive",                                                            L"https" },
+	{ DROPBOX,      L"dropbox",  true, 443, false, "Dropbox",                                                                 L"https" },
+	{ ONEDRIVE,     L"onedrive", true, 443, false, "Microsoft OneDrive",                                                      L"https" },
+	{ B2,           L"b2",       true, 443, false, "Backblaze B2",                                                            L"https" },
+	{ UNKNOWN,      L"",         false, 21, false, "", L"" }
 };
 
 static std::vector<ServerProtocol> const defaultProtocols = {
@@ -260,7 +262,12 @@ CServer::CServer(ServerProtocol protocol, ServerType type, std::wstring const& h
 	m_protocol = protocol;
 	m_type = type;
 	m_host = host;
-	m_port = port;
+	if (port) {
+		m_port = port;
+	}
+	else {
+		port = GetDefaultPort(protocol);
+	}
 }
 
 void CServer::SetType(ServerType type)
@@ -566,6 +573,11 @@ bool CServer::GetBypassProxy() const
 	return m_bypassProxy;
 }
 
+bool CServer::HasFeature(ProtocolFeature const feature) const
+{
+	return ProtocolHasFeature(m_protocol, feature);
+}
+
 bool CServer::ProtocolHasFeature(ServerProtocol const protocol, ProtocolFeature const feature)
 {
 	switch (feature) {
@@ -601,7 +613,13 @@ bool CServer::ProtocolHasFeature(ServerProtocol const protocol, ProtocolFeature
 		}
 		break;
 	case ProtocolFeature::RecursiveDelete:
-		if (protocol == GOOGLE_DRIVE || protocol == DROPBOX) {
+		if (protocol == GOOGLE_DRIVE || protocol == DROPBOX || protocol == ONEDRIVE) {
+			return true;
+		}
+		break;
+	case ProtocolFeature::ServerAssignedHome:
+		if (protocol == FTP || protocol == FTPS || protocol == FTPES || protocol == INSECURE_FTP ||
+			protocol == SFTP) {
 			return true;
 		}
 		break;
@@ -780,12 +798,14 @@ std::vector<LogonType> GetSupportedLogonTypes(ServerProtocol protocol)
 	case AZURE_FILE:
 	case AZURE_BLOB:
 	case SWIFT:
+	case B2:
 		return {LogonType::normal, LogonType::ask};
 	case WEBDAV:
 		return {LogonType::anonymous, LogonType::normal, LogonType::ask};
 	case GOOGLE_CLOUD:
 	case GOOGLE_DRIVE:
 	case DROPBOX:
+	case ONEDRIVE:
 		return {LogonType::interactive};
 	case HTTPS:
 	case UNKNOWN:
@@ -841,6 +861,10 @@ std::tuple<std::wstring, std::wstring> GetDefaultHost(ServerProtocol protocol)
 		return std::tuple<std::wstring, std::wstring>{L"s3.amazonaws.com", L""};
 	case DROPBOX:
 		return std::tuple<std::wstring, std::wstring>{L"api.dropboxapi.com", L""};
+	case ONEDRIVE:
+		return std::tuple<std::wstring, std::wstring>{L"graph.microsoft.com", L""};
+	case B2:
+		return std::tuple<std::wstring, std::wstring>{L"api.backblazeb2.com", L""};
 	default:
 		break;
 	}
@@ -850,5 +874,28 @@ std::tuple<std::wstring, std::wstring> GetDefaultHost(ServerProtocol protocol)
 
 bool ProtocolHasUser(ServerProtocol protocol)
 {
-	return protocol != GOOGLE_DRIVE &&  protocol != DROPBOX;
+	return protocol != DROPBOX && protocol != ONEDRIVE;
+}
+
+bool CServer::SameResource(CServer const& other) const
+{
+	// We include post-login commands as it may be used for things like the HOST command.
+	// We include proxy parameters as a hostname may resolve to different servers depending on whether a proxy is used.
+
+	auto l = std::tie(m_protocol, m_host, m_port, m_user, m_postLoginCommands, m_bypassProxy, extraParameters_);
+	auto r = std::tie(other.m_protocol, other.m_host, other.m_port, other.m_user, other.m_postLoginCommands, other.m_bypassProxy, other.extraParameters_);
+
+	return l == r;
+}
+
+bool CServer::SameContent(CServer const& other) const
+{
+	if (!SameResource(other)) {
+		return false;
+	}
+
+	auto l = std::tie(m_timezoneOffset, m_encodingType, m_customEncoding);
+	auto r = std::tie(other.m_timezoneOffset, other.m_encodingType, other.m_customEncoding);
+
+	return l == r;
 }
