@@ -11,22 +11,20 @@
 #include "welcome_dialog.h"
 #include "msgbox.h"
 #include "themeprovider.h"
+#include "wxfilesystem_blob_handler.h"
 
 #include <libfilezilla/local_filesys.hpp>
 
 #include <wx/evtloop.h>
 
-#include "wxfilesystem_blob_handler.h"
-#include "xrc_helper.h"
-#ifdef __WXMSW__
-#include <wx/dynlib.h>
-#endif
 #ifdef WITH_LIBDBUS
 #include <../dbus/session_manager.h>
 #endif
 
 #if defined(__WXMAC__) || defined(__UNIX__)
 #include <wx/stdpaths.h>
+#elif defined(__WXMSW__)
+#include <shobjidl.h>
 #endif
 
 #include "locale_initializer.h"
@@ -63,32 +61,6 @@ CFileZillaApp::~CFileZillaApp()
 
 #ifdef __WXMSW__
 namespace {
-extern "C"
-{
-	typedef HRESULT(WINAPI *t_SetCurrentProcessExplicitAppUserModelID)(PCWSTR AppID);
-}
-
-static void SetAppId()
-{
-	wxDynamicLibrary dll;
-	if (!dll.Load(_T("shell32.dll"))) {
-		return;
-	}
-
-	if (!dll.HasSymbol(_T("SetCurrentProcessExplicitAppUserModelID"))) {
-		return;
-	}
-
-	t_SetCurrentProcessExplicitAppUserModelID pSetCurrentProcessExplicitAppUserModelID =
-		(t_SetCurrentProcessExplicitAppUserModelID)dll.GetSymbol(_T("SetCurrentProcessExplicitAppUserModelID"));
-
-	if (!pSetCurrentProcessExplicitAppUserModelID) {
-		return;
-	}
-
-	pSetCurrentProcessExplicitAppUserModelID(_T("FileZilla.Client.AppID"));
-}
-
 static bool InitWinsock()
 {
 	WSADATA d{};
@@ -201,7 +173,7 @@ bool CFileZillaApp::OnInit()
 		return false;
 	}
 
-	SetAppId();
+	SetCurrentProcessExplicitAppUserModelID(L"FileZilla.Client.AppID");
 #endif
 
 	//wxSystemOptions is slow, if a value is not set, it keeps querying the environment
@@ -474,25 +446,6 @@ bool CFileZillaApp::LoadResourceFiles()
 
 	m_resourceDir.AddSegment(_T("resources"));
 
-	wxXmlResource *pResource = wxXmlResource::Get();
-
-#ifndef __WXDEBUG__
-	pResource->SetFlags(pResource->GetFlags() | wxXRC_NO_RELOADING);
-#endif
-
-	InitHandlers(*pResource);
-
-	fz::local_filesys fs;
-	std::wstring dir = m_resourceDir.GetPath() + L"xrc/";
-	pResource->LoadFile(wxString(dir + L"certificate.xrc"));
-	pResource->LoadFile(wxString(dir + L"dialogs.xrc"));
-	pResource->LoadFile(wxString(dir + L"inputdialog.xrc"));
-	pResource->LoadFile(wxString(dir + L"netconfwizard.xrc"));
-	pResource->LoadFile(wxString(dir + L"settings.xrc"));
-	pResource->LoadFile(wxString(dir + L"sitemanager.xrc"));
-	pResource->LoadFile(wxString(dir + L"update.xrc"));
-	pResource->LoadFile(wxString(dir + L"storj.xrc"));
-
 	// Useful for XRC files with embedded image data.
 	wxFileSystem::AddHandler(new wxFileSystemBlobHandler);
 
@@ -806,6 +759,7 @@ void CFileZillaApp::AddStartupProfileRecord(std::string const& msg)
 void CFileZillaApp::ShowStartupProfile()
 {
 	if (m_profile_start && m_pCommandLine && m_pCommandLine->HasSwitch(CCommandLine::debug_startup)) {
+		AddStartupProfileRecord("CFileZillaApp::ShowStartupProfile");
 		wxString msg = _T("Profile:\n");
 		for (auto const& p : m_startupProfile) {
 			auto const diff = p.first - m_profile_start;
