@@ -114,7 +114,7 @@ public:
 				return wxDragNone;
 			}
 
-			if (!m_pLocalListView->m_state.GetServer() || m_pRemoteDataObject->GetServer().server != m_pLocalListView->m_state.GetServer().server) {
+			if (!m_pLocalListView->m_state.GetSite() || m_pRemoteDataObject->GetSite().server != m_pLocalListView->m_state.GetSite().server) {
 				wxMessageBoxEx(_("Drag&drop between different servers has not been implemented yet."));
 				return wxDragNone;
 			}
@@ -471,11 +471,11 @@ int CLocalListView::OnGetItemImage(long item) const
 	int &icon = data->icon;
 
 	if (icon == -2) {
-		wxString path;
-		if (data->name != _T("..")) {
+		std::wstring path;
+		if (data->name != L"..") {
 #ifdef __WXMSW__
-			if (m_dir.GetPath() == _T("\\")) {
-				path = data->name + _T("\\");
+			if (m_dir.GetPath() == L"\\") {
+				path = data->name + L"\\";
 			}
 			else
 #endif
@@ -572,8 +572,8 @@ void CLocalListView::OnItemActivated(wxListEvent &event)
 		return;
 	}
 
-	ServerWithCredentials const& server = m_state.GetServer();
-	if (!server) {
+	Site const& site = m_state.GetSite();
+	if (!site) {
 		wxBell();
 		return;
 	}
@@ -586,7 +586,7 @@ void CLocalListView::OnItemActivated(wxListEvent &event)
 
 	const bool queue_only = action == 1;
 
-	m_pQueue->QueueFile(queue_only, false, data->name, wxEmptyString, m_dir, path, server, data->size);
+	m_pQueue->QueueFile(queue_only, false, data->name, wxEmptyString, m_dir, path, site, data->size);
 	m_pQueue->QueueFile_Finish(true);
 }
 
@@ -625,11 +625,11 @@ void CLocalListView::DisplayDrives()
 {
 	int count = m_fileData.size();
 
-	std::list<std::wstring> drives = CVolumeDescriptionEnumeratorThread::GetDrives();
+	std::vector<std::wstring> drives = CVolumeDescriptionEnumeratorThread::GetDrives();
 	for (auto it = drives.begin(); it != drives.end(); ++it) {
-		wxString drive = *it;
-		if (drive.Right(1) == _T("\\")) {
-			drive.RemoveLast();
+		std::wstring drive = *it;
+		if (!drive.empty() && drive.back() == '\\') {
+			drive.pop_back();
 		}
 
 		CLocalFileData data;
@@ -637,11 +637,11 @@ void CLocalListView::DisplayDrives()
 		data.label = fz::sparse_optional<std::wstring>(data.name);
 		data.dir = true;
 		data.size = -1;
-		data.icon = GetIconIndex(iconType::dir, wxString(), false);
+		data.icon = GetIconIndex(iconType::dir, std::wstring(), false);
 
 		m_fileData.push_back(data);
 		m_indexMapping.push_back(count);
-		count++;
+		++count;
 	}
 
 	if (m_pFilelistStatusBar) {
@@ -860,8 +860,8 @@ void CLocalListView::OnContextMenu(wxContextMenuEvent& event)
 
 void CLocalListView::OnMenuUpload(wxCommandEvent& event)
 {
-	ServerWithCredentials const& server = m_state.GetServer();
-	if (!server) {
+	Site const& site = m_state.GetSite();
+	if (!site) {
 		wxBell();
 		return;
 	}
@@ -915,7 +915,7 @@ void CLocalListView::OnMenuUpload(wxCommandEvent& event)
 			root.add_dir_to_visit(localPath, remotePath);
 		}
 		else {
-			m_pQueue->QueueFile(queue_only, false, data->name, wxEmptyString, m_dir, remotePath, server, data->size);
+			m_pQueue->QueueFile(queue_only, false, data->name, wxEmptyString, m_dir, remotePath, site, data->size);
 			added = true;
 		}
 	}
@@ -1133,9 +1133,9 @@ bool CLocalListView::OnAcceptRename(const wxListEvent& event)
 		return false;
 	}
 
-	wxString newname = event.GetLabel();
+	std::wstring newname = event.GetLabel().ToStdWstring();
 #ifdef __WXMSW__
-	newname = newname.Left(255);
+	newname = newname.substr(0, 255);
 #endif
 
 	if (newname == data->name) {
@@ -1298,7 +1298,7 @@ void CLocalListView::ReselectItems(const std::vector<std::wstring>& selectedName
 
 		if (focusedItem != -1 && GetItemCount() != 0) {
 			if (focusedItem >= GetItemCount()) {
-				--focusedItem;
+				focusedItem = GetItemCount() - 1;
 			}
 			SetItemState(focusedItem, wxLIST_STATE_FOCUSED, wxLIST_STATE_FOCUSED);
 		}
@@ -1362,7 +1362,7 @@ void CLocalListView::ReselectItems(const std::vector<std::wstring>& selectedName
 	}
 }
 
-void CLocalListView::OnStateChange(t_statechange_notifications notification, const wxString& data, const void*)
+void CLocalListView::OnStateChange(t_statechange_notifications notification, std::wstring const& data, const void*)
 {
 	if (notification == STATECHANGE_LOCAL_DIR) {
 		DisplayDir(m_state.GetLocalDir());
@@ -1455,7 +1455,7 @@ void CLocalListView::OnBeginDrag(wxListEvent&)
 	}
 }
 
-void CLocalListView::RefreshFile(const wxString& file)
+void CLocalListView::RefreshFile(std::wstring const& file)
 {
 	CLocalFileData data;
 
@@ -1661,7 +1661,7 @@ void CLocalListView::StartComparison()
 	}
 }
 
-bool CLocalListView::get_next_file(wxString& name, bool& dir, int64_t& size, fz::datetime& date)
+bool CLocalListView::get_next_file(std::wstring & name, bool& dir, int64_t& size, fz::datetime& date)
 {
 	if (++m_comparisonIndex >= (int)m_originalIndexMapping.size()) {
 		return false;
@@ -1672,7 +1672,7 @@ bool CLocalListView::get_next_file(wxString& name, bool& dir, int64_t& size, fz:
 		return false;
 	}
 
-	const CLocalFileData& data = m_fileData[index];
+	CLocalFileData const& data = m_fileData[index];
 
 	name = data.name;
 	dir = data.dir;
@@ -1748,17 +1748,17 @@ wxString CLocalListView::GetItemText(int item, unsigned int column)
 
 void CLocalListView::OnMenuEdit(wxCommandEvent&)
 {
-	ServerWithCredentials server;
+	Site site;
 	CServerPath path;
 
-	if (!m_state.GetServer()) {
+	if (!m_state.GetSite()) {
 		if (COptions::Get()->GetOptionVal(OPTION_EDIT_TRACK_LOCAL)) {
 			wxMessageBoxEx(_("Cannot edit file, not connected to any server."), _("Editing failed"), wxICON_EXCLAMATION);
 			return;
 		}
 	}
 	else {
-		server = m_state.GetServer();
+		site = m_state.GetSite();
 
 		path = m_state.GetRemotePath();
 		if (path.empty()) {
@@ -1794,7 +1794,7 @@ void CLocalListView::OnMenuEdit(wxCommandEvent&)
 	}
 
 	CEditHandler* pEditHandler = CEditHandler::Get();
-	pEditHandler->Edit(CEditHandler::local, selected_item, path, server, this);
+	pEditHandler->Edit(CEditHandler::local, selected_item, path, site, this);
 }
 
 void CLocalListView::OnMenuOpen(wxCommandEvent&)
@@ -1907,8 +1907,7 @@ void CLocalListView::OnVolumesEnumerated(wxCommandEvent& event)
 		return;
 	}
 
-	std::list<CVolumeDescriptionEnumeratorThread::t_VolumeInfo> volumeInfo;
-	volumeInfo = volumeEnumeratorThread_->GetVolumes();
+	std::vector<CVolumeDescriptionEnumeratorThread::t_VolumeInfo> volumeInfo = volumeEnumeratorThread_->GetVolumes();
 
 	if (event.GetEventType() == fzEVT_VOLUMESENUMERATED) {
 		volumeEnumeratorThread_.reset();
@@ -1918,8 +1917,8 @@ void CLocalListView::OnVolumesEnumerated(wxCommandEvent& event)
 		return;
 	}
 
-	for (std::list<CVolumeDescriptionEnumeratorThread::t_VolumeInfo>::const_iterator iter = volumeInfo.begin(); iter != volumeInfo.end(); ++iter) {
-		std::wstring drive = iter->volume;
+	for (auto const& info : volumeInfo) {
+		std::wstring const& drive = info.volume;
 
 		unsigned int item, index;
 		for (item = m_hasParent ? 1 : 0; item < m_indexMapping.size(); ++item) {
@@ -1932,11 +1931,11 @@ void CLocalListView::OnVolumesEnumerated(wxCommandEvent& event)
 			continue;
 		}
 
-		if (!iter->volumeName.empty()) {
-			m_fileData[index].label = fz::sparse_optional<std::wstring>(drive + _T(" (") + iter->volumeName + _T(")"));
+		if (!info.volumeName.empty()) {
+			m_fileData[index].label = fz::sparse_optional<std::wstring>(drive + _T(" (") + info.volumeName + _T(")"));
 		}
-		if (iter->icon != -1) {
-			m_fileData[index].icon = iter->icon;
+		if (info.icon != -1) {
+			m_fileData[index].icon = info.icon;
 		}
 
 		RefreshItem(item);
