@@ -36,7 +36,7 @@ public:
 	virtual ~CLocalListView();
 
 protected:
-	void OnStateChange(t_statechange_notifications notification, const wxString& data, const void*);
+	void OnStateChange(t_statechange_notifications notification, std::wstring const& data, const void*) override;
 	bool DisplayDir(CLocalPath const& dirname);
 	void ApplyCurrentFilter();
 
@@ -63,7 +63,7 @@ protected:
 public:
 	virtual bool CanStartComparison();
 	virtual void StartComparison();
-	virtual bool get_next_file(wxString& name, bool &dir, int64_t &size, fz::datetime& date);
+	virtual bool get_next_file(std::wstring & name, bool &dir, int64_t &size, fz::datetime& date) override;
 	virtual void FinishComparison();
 
 	virtual bool ItemIsDir(int index) const;
@@ -77,7 +77,7 @@ protected:
 
 	virtual std::unique_ptr<CFileListCtrlSortBase> GetSortComparisonObject() override;
 
-	void RefreshFile(const wxString& file);
+	void RefreshFile(std::wstring const& file);
 
 	virtual void OnNavigationEvent(bool forward);
 
