@@ -199,11 +199,11 @@ public:
 	}
 
 protected:
-	virtual void OnStateChange(CState* pState, t_statechange_notifications notification, const wxString&, const void*)
+	virtual void OnStateChange(CState* pState, t_statechange_notifications notification, std::wstring const&, const void*) override
 	{
 		if (notification == STATECHANGE_CHANGEDCONTEXT) {
 			// Update window title
-			if (!pState || !pState->GetServer()) {
+			if (!pState || !pState->GetSite()) {
 				m_pMainFrame->SetTitle(_T("FileZilla"));
 			}
 			else {
@@ -235,10 +235,9 @@ protected:
 		}
 
 		if (notification == STATECHANGE_SERVER) {
-			ServerWithCredentials const& server = pState->GetServer();
-
 			if (pState == CContextManager::Get()->GetCurrentContext()) {
-				if (!server) {
+				Site const& site = pState->GetSite();
+				if (!site) {
 					m_pMainFrame->SetTitle(_T("FileZilla"));
 				}
 				else {
@@ -328,8 +327,6 @@ CMainFrame::CMainFrame()
 	SetIcons(CThemeProvider::GetIconBundle(_T("ART_FILEZILLA")));
 #endif
 
-	m_pThemeProvider = new CThemeProvider();
-
 	CPowerManagement::Create(this);
 
 	// It's important that the context control gets created before our own state handler
@@ -462,6 +459,7 @@ CMainFrame::CMainFrame()
 		m_pBottomSplitter->Initialize(m_pContextControl);
 	}
 
+	wxGetApp().AddStartupProfileRecord("CMainFrame::CMainFrame pre layout");
 	m_pWindowStateManager = new CWindowStateManager(this);
 	m_pWindowStateManager->Restore(OPTION_MAINWINDOW_POSITION);
 
@@ -609,16 +607,16 @@ void CMainFrame::OnMenuHandler(wxCommandEvent &event)
 		OpenSiteManager();
 	}
 	else if (event.GetId() == XRCID("ID_MENU_FILE_COPYSITEMANAGER")) {
-		ServerWithCredentials server;
+		Site site;
 		CState* pState = CContextManager::Get()->GetCurrentContext();
 		if (pState) {
-			server = pState->GetServer();
+			site = pState->GetSite();
 		}
-		if (!server) {
+		if (!site) {
 			wxMessageBoxEx(_("Not connected to any server."), _("Cannot add server to Site Manager"), wxICON_EXCLAMATION);
 			return;
 		}
-		OpenSiteManager(&server);
+		OpenSiteManager(&site);
 	}
 	else if (event.GetId() == XRCID("ID_MENU_SERVER_CMD")) {
 		CState* pState = CContextManager::Get()->GetCurrentContext();
@@ -852,7 +850,7 @@ void CMainFrame::OnMenuHandler(wxCommandEvent &event)
 		}
 
 		Site const old_site = pState->GetSite();
-		std::wstring sitePath = old_site.m_path;
+		std::wstring sitePath = old_site.SitePath();
 
 		CContextControl::_context_controls* controls = m_pContextControl->GetCurrentControls();
 		if (!controls) {
@@ -862,11 +860,11 @@ void CMainFrame::OnMenuHandler(wxCommandEvent &event)
 		// controls->last_bookmark_path can get modified if it's empty now
 		int res;
 		if (event.GetId() == XRCID("ID_BOOKMARK_ADD")) {
-			CNewBookmarkDialog dlg(this, sitePath, old_site.server_ ? &old_site.server_ : 0);
+			CNewBookmarkDialog dlg(this, sitePath, old_site ? &old_site : 0);
 			res = dlg.Run(pState->GetLocalDir().GetPath(), pState->GetRemotePath());
 		}
 		else {
-			CBookmarksDialog dlg(this, sitePath, old_site.server_ ? &old_site.server_ : 0);
+			CBookmarksDialog dlg(this, sitePath, old_site ? &old_site : 0);
 			res = dlg.Run();
 		}
 		if (res == wxID_OK) {
@@ -876,7 +874,7 @@ void CMainFrame::OnMenuHandler(wxCommandEvent &event)
 					for (int i = 0; i < m_pContextControl->GetTabCount(); ++i) {
 						CContextControl::_context_controls *tab_controls = m_pContextControl->GetControlsFromTabIndex(i);
 						if (tab_controls) {
-							tab_controls->pState->UpdateSite(old_site.m_path, *site);
+							tab_controls->pState->UpdateSite(old_site.SitePath(), *site);
 						}
 					}
 				}
@@ -1193,7 +1191,7 @@ bool CMainFrame::CloseDialogsAndQuit(wxCloseEvent &event)
 			}
 
 			wxDialog* pDialog = dynamic_cast<wxDialog*>(pTop);
-			if (pDialog) {
+			if (pDialog && pDialog->IsModal()) {
 				pDialog->EndModal(wxID_CANCEL);
 			}
 			else {
@@ -1365,8 +1363,12 @@ void CMainFrame::OnClose(wxCloseEvent &event)
 	if (m_pContextControl) {
 		CContextControl::_context_controls* controls = m_pContextControl->GetCurrentControls();
 		if (controls) {
-			controls->pLocalListView->SaveColumnSettings(OPTION_LOCALFILELIST_COLUMN_WIDTHS, OPTION_LOCALFILELIST_COLUMN_SHOWN, OPTION_LOCALFILELIST_COLUMN_ORDER);
-			controls->pRemoteListView->SaveColumnSettings(OPTION_REMOTEFILELIST_COLUMN_WIDTHS, OPTION_REMOTEFILELIST_COLUMN_SHOWN, OPTION_REMOTEFILELIST_COLUMN_ORDER);
+			if (controls->pLocalListView) {
+				controls->pLocalListView->SaveColumnSettings(OPTION_LOCALFILELIST_COLUMN_WIDTHS, OPTION_LOCALFILELIST_COLUMN_SHOWN, OPTION_LOCALFILELIST_COLUMN_ORDER);
+			}
+			if (controls->pRemoteListView) {
+				controls->pRemoteListView->SaveColumnSettings(OPTION_REMOTEFILELIST_COLUMN_WIDTHS, OPTION_REMOTEFILELIST_COLUMN_SHOWN, OPTION_REMOTEFILELIST_COLUMN_ORDER);
+			}
 		}
 
 		m_pContextControl->SaveTabs();
@@ -1410,7 +1412,11 @@ void CMainFrame::OnRefresh(wxCommandEvent &)
 		return;
 	}
 
-	pState->RefreshRemote();
+	bool clear_cache = false;
+	if (wxGetKeyState(WXK_CONTROL)) {
+		clear_cache = true;
+	}
+	pState->RefreshRemote(clear_cache);
 	pState->RefreshLocal();
 }
 
@@ -1434,7 +1440,7 @@ void CMainFrame::OnTimer(wxTimerEvent& event)
 #endif
 }
 
-void CMainFrame::OpenSiteManager(ServerWithCredentials const* pServer)
+void CMainFrame::OpenSiteManager(Site const* site)
 {
 	CState* pState = CContextManager::Get()->GetCurrentContext();
 	if (!pState) {
@@ -1443,55 +1449,55 @@ void CMainFrame::OpenSiteManager(ServerWithCredentials const* pServer)
 
 	CSiteManagerDialog dlg;
 
-	std::set<std::wstring> handled_paths;
 	std::vector<CSiteManagerDialog::_connected_site> connected_sites;
 
-	if (pServer) {
+	if (site) {
 		CSiteManagerDialog::_connected_site connected_site;
-		connected_site.server = *pServer;
+		connected_site.site = *site;
 		connected_sites.push_back(connected_site);
 	}
 
 	for (int i = 0; i < m_pContextControl->GetTabCount(); ++i) {
 		CContextControl::_context_controls *controls =  m_pContextControl->GetControlsFromTabIndex(i);
-		if (!controls) {
+		if (!controls || !controls->pState) {
 			continue;
 		}
 
-		Site const& site = controls->pState->GetSite();
-		std::wstring const& path = site.m_path;
+		Site site = controls->pState->GetSite();
+		if (!site) {
+			site = controls->pState->GetLastSite();
+		}
+
+		std::wstring const& path = site.SitePath();
 		if (path.empty()) {
 			continue;
 		}
-		if (handled_paths.find(path) != handled_paths.end()) {
+
+		auto it = std::find_if(connected_sites.cbegin(), connected_sites.cend(), [&](auto const& v) { return v.old_path == path; });
+		if (it != connected_sites.cend()) {
 			continue;
 		}
 
 		CSiteManagerDialog::_connected_site connected_site;
 		connected_site.old_path = path;
-		connected_site.server = site.server_;
+		connected_site.site = site;
 		connected_sites.push_back(connected_site);
-		handled_paths.insert(path);
 	}
 
-	if (!dlg.Create(this, &connected_sites, pServer)) {
+	if (!dlg.Create(this, &connected_sites, site)) {
 		return;
 	}
 
 	int res = dlg.ShowModal();
 	if (res == wxID_YES || res == wxID_OK) {
 		// Update bookmark paths
-		for (auto const& connected_site : connected_sites) {
-			std::unique_ptr<Site> site = CSiteManager::GetSiteByPath(connected_site.new_path, false).first;
-			if (site) {
-				for (int i = 0; i < m_pContextControl->GetTabCount(); ++i) {
-					CContextControl::_context_controls *controls = m_pContextControl->GetControlsFromTabIndex(i);
-					if (!controls) {
-						continue;
-					}
-					controls->pState->UpdateSite(connected_site.old_path, *site);
-				}
+		for (int i = 0; i < m_pContextControl->GetTabCount(); ++i) {
+			CContextControl::_context_controls *controls = m_pContextControl->GetControlsFromTabIndex(i);
+			if (!controls || !controls->pState) {
+				continue;
 			}
+
+			controls->pState->UpdateKnownSites(connected_sites);
 		}
 	}
 
@@ -2023,7 +2029,7 @@ void CMainFrame::OnSitemanagerDropdown(wxCommandEvent& event)
 bool CMainFrame::ConnectToSite(Site & data, Bookmark const& bookmark, CState* pState)
 {
 	// First check if we need to ask user for a password
-	if (!CLoginManager::Get().GetPassword(data.server_, false, data.server_.server.GetName())) {
+	if (!CLoginManager::Get().GetPassword(data, false)) {
 		return false;
 	}
 
@@ -2081,7 +2087,7 @@ bool CMainFrame::ConnectToSite(Site & data, Bookmark const& bookmark, CState* pS
 	// Apply comparison and sync browsing options
 	// FIXME: Move to state?
 	if (!bookmark.m_localDir.empty()) {
-		bool set = pState->SetLocalDir(bookmark.m_localDir.ToStdWstring(), 0, false);
+		bool set = pState->SetLocalDir(bookmark.m_localDir, 0, false);
 
 		if (set && bookmark.m_sync) {
 			wxASSERT(!bookmark.m_remoteDir.empty());
@@ -2351,7 +2357,7 @@ void CMainFrame::OnToolbarComparison(wxCommandEvent&)
 			return;
 		}
 
-		if (controls->pLocalListSearchPanel->IsShown() || controls->pRemoteListSearchPanel->IsShown()) {
+		if ((controls->pLocalListSearchPanel && controls->pLocalListSearchPanel->IsShown()) || (controls->pRemoteListSearchPanel && controls->pRemoteListSearchPanel->IsShown())) {
 			CConditionalDialog dlg(this, CConditionalDialog::quick_search, CConditionalDialog::yesno);
 			dlg.SetTitle(_("Directory comparison"));
 			dlg.AddText(_("To compare directories quick search must be closed."));
@@ -2525,34 +2531,32 @@ void CMainFrame::ProcessCommandLine()
 		}
 	}
 
-	std::wstring param = pCommandLine->GetParameter().ToStdWstring();
+	std::wstring param = pCommandLine->GetParameter();
 	if (!param.empty()) {
 		std::wstring error;
 
-		ServerWithCredentials server;
+		Site site;
 
 		wxString logontype = pCommandLine->GetOption(CCommandLine::logontype);
 		if (logontype == _T("ask")) {
-			server.SetLogonType(LogonType::ask);
+			site.SetLogonType(LogonType::ask);
 		}
 		else if (logontype == _T("interactive")) {
-			server.SetLogonType(LogonType::interactive);
+			site.SetLogonType(LogonType::interactive);
 		}
 
 		CServerPath path;
-		if (!server.ParseUrl(param, 0, std::wstring(), std::wstring(), error, path)) {
+		if (!site.ParseUrl(param, 0, std::wstring(), std::wstring(), error, path)) {
 			wxString str = _("Parameter not a valid URL");
 			str += _T("\n") + error;
 			wxMessageBoxEx(error, _("Syntax error in command line"));
 		}
 
-		if (COptions::Get()->GetOptionVal(OPTION_DEFAULT_KIOSKMODE) && server.credentials.logonType_ == LogonType::normal) {
-			server.SetLogonType(LogonType::ask);
-			CLoginManager::Get().RememberPassword(server);
+		if (COptions::Get()->GetOptionVal(OPTION_DEFAULT_KIOSKMODE) && site.credentials.logonType_ == LogonType::normal) {
+			site.SetLogonType(LogonType::ask);
+			CLoginManager::Get().RememberPassword(site);
 		}
 
-		Site site;
-		site.server_ = server;
 		Bookmark bm;
 		bm.m_remoteDir = path;
 		ConnectToSite(site, bm);
@@ -2838,6 +2842,7 @@ void CMainFrame::SetupKeyboardAccelerators()
 	}
 	entries.emplace_back(wxACCEL_CMD | wxACCEL_SHIFT, 'O', m_comparisonToggleAcceleratorId);
 	entries.emplace_back(wxACCEL_CMD | wxACCEL_SHIFT, 'I', XRCID("ID_MENU_VIEW_FILTERS"));
+	entries.emplace_back(wxACCEL_CMD, WXK_F5, XRCID("ID_REFRESH"));
 #ifdef __WXMAC__
 	entries.emplace_back(wxACCEL_CMD, ',', XRCID("wxID_PREFERENCES"));
 
