@@ -99,14 +99,13 @@ public:
 				return wxDragNone;
 			}
 
-			if (!m_pRemoteTreeView->m_state.GetServer() || m_pRemoteDataObject->GetServer().server != m_pRemoteTreeView->m_state.GetServer().server) {
+			if (!m_pRemoteTreeView->m_state.GetSite() || m_pRemoteDataObject->GetSite().server != m_pRemoteTreeView->m_state.GetSite().server) {
 				wxMessageBoxEx(_("Drag&drop between different servers has not been implemented yet."));
 				return wxDragNone;
 			}
 
 			// Make sure path path is valid
-			if (path == m_pRemoteDataObject->GetServerPath())
-			{
+			if (path == m_pRemoteDataObject->GetServerPath()) {
 				wxMessageBoxEx(_("Source and path of the drop operation are identical"));
 				return wxDragNone;
 			}
@@ -278,7 +277,7 @@ CRemoteTreeView::~CRemoteTreeView()
 	delete m_pImageList;
 }
 
-void CRemoteTreeView::OnStateChange(t_statechange_notifications notification, const wxString&, const void* data2)
+void CRemoteTreeView::OnStateChange(t_statechange_notifications notification, std::wstring const&, const void* data2)
 {
 	if (notification == STATECHANGE_REMOTE_DIR) {
 		SetDirectoryListing(m_state.GetRemoteDir(), data2 ? *reinterpret_cast<bool const*>(data2) : true);
@@ -512,13 +511,14 @@ bool CRemoteTreeView::HasSubdirs(const CDirectoryListing& listing, const CFilter
 		return true;
 	}
 
-	const wxString path = listing.path.GetPath();
-	for (unsigned int i = 0; i < listing.GetCount(); ++i) {
-		if (!listing[i].is_dir()) {
+	std::wstring const path = listing.path.GetPath();
+	for (size_t i = 0; i < listing.size(); ++i) {
+		auto const& entry = listing[i];
+		if (!entry.is_dir()) {
 			continue;
 		}
 
-		if (filter.FilenameFiltered(listing[i].name, path, true, -1, false, 0, listing[i].time)) {
+		if (filter.FilenameFiltered(entry.name, path, true, -1, false, 0, entry.time)) {
 			continue;
 		}
 
@@ -532,19 +532,20 @@ void CRemoteTreeView::DisplayItem(wxTreeItemId parent, const CDirectoryListing&
 {
 	DeleteChildren(parent);
 
-	const wxString path = listing.path.GetPath();
+	std::wstring const path = listing.path.GetPath();
 
 	CFilterDialog filter;
-	for (unsigned int i = 0; i < listing.GetCount(); ++i) {
-		if (!listing[i].is_dir()) {
+	for (size_t i = 0; i < listing.size(); ++i) {
+		auto const& entry = listing[i];
+		if (!entry.is_dir()) {
 			continue;
 		}
 
-		if (filter.FilenameFiltered(listing[i].name, path, true, -1, false, 0, listing[i].time)) {
+		if (filter.FilenameFiltered(entry.name, path, true, -1, false, 0, entry.time)) {
 			continue;
 		}
 
-		std::wstring const& name = listing[i].name;
+		std::wstring const& name = entry.name;
 		CServerPath subdir = listing.path;
 		subdir.AddSegment(name);
 
@@ -555,7 +556,7 @@ void CRemoteTreeView::DisplayItem(wxTreeItemId parent, const CDirectoryListing&
 			SetItemImages(child, false);
 
 			if (HasSubdirs(subListing, filter)) {
-				AppendItem(child, _T(""), -1, -1);
+				AppendItem(child, L"", -1, -1);
 			}
 		}
 		else {
@@ -579,16 +580,17 @@ void CRemoteTreeView::RefreshItem(wxTreeItemId parent, const CDirectoryListing&
 
 	CFilterManager filter;
 
-	wxString const path = listing.path.GetPath();
+	std::wstring const path = listing.path.GetPath();
 
 	std::vector<std::wstring> dirs;
-	for (unsigned int i = 0; i < listing.GetCount(); ++i) {
-		if (!listing[i].is_dir()) {
+	for (size_t i = 0; i < listing.size(); ++i) {
+		auto const& entry = listing[i];
+		if (!entry.is_dir()) {
 			continue;
 		}
 
-		if (!filter.FilenameFiltered(listing[i].name, path, true, -1, false, 0, listing[i].time)) {
-			dirs.push_back(listing[i].name);
+		if (!filter.FilenameFiltered(entry.name, path, true, -1, false, 0, entry.time)) {
+			dirs.push_back(entry.name);
 		}
 	}
 
@@ -622,11 +624,13 @@ void CRemoteTreeView::RefreshItem(wxTreeItemId parent, const CDirectoryListing&
 		else if (cmp > 0) {
 			// Child no longer exists
 			wxTreeItemId sel = GetSelection();
-			while (sel && sel != child)
+			while (sel && sel != child) {
 				sel = GetItemParent(sel);
+			}
 			wxTreeItemId prev = GetPrevSibling(child);
-			if (!sel || will_select_parent)
+			if (!sel || will_select_parent) {
 				Delete(child);
+			}
 			child = prev;
 		}
 		else if (cmp < 0) {
@@ -659,11 +663,13 @@ void CRemoteTreeView::RefreshItem(wxTreeItemId parent, const CDirectoryListing&
 	while (child) {
 		// Child no longer exists
 		wxTreeItemId sel = GetSelection();
-		while (sel && sel != child)
+		while (sel && sel != child) {
 			sel = GetItemParent(sel);
+		}
 		wxTreeItemId prev = GetPrevSibling(child);
-		if (!sel || will_select_parent)
+		if (!sel || will_select_parent) {
 			Delete(child);
+		}
 		child = prev;
 	}
 	while (iter != dirs.rend()) {
@@ -826,12 +832,12 @@ void CRemoteTreeView::OnBeginDrag(wxTreeEvent& event)
 
 	wxDataObjectComposite object;
 
-	ServerWithCredentials server = m_state.GetServer(); // Make a copy as DoDragDrop later runs the event loop
-	if (!server) {
+	Site site = m_state.GetSite(); // Make a copy as DoDragDrop later runs the event loop
+	if (!site) {
 		return;
 	}
 
-	CRemoteDataObject *pRemoteDataObject = new CRemoteDataObject(server, parent);
+	CRemoteDataObject *pRemoteDataObject = new CRemoteDataObject(site, parent);
 	pRemoteDataObject->AddFile(lastSegment, true, -1, false);
 
 	pRemoteDataObject->Finalize();
@@ -854,7 +860,7 @@ void CRemoteTreeView::OnBeginDrag(wxTreeEvent& event)
 
 	CDragDropManager* pDragDropManager = CDragDropManager::Init();
 	pDragDropManager->pDragSource = this;
-	pDragDropManager->server = server;
+	pDragDropManager->site = site;
 	pDragDropManager->remoteParent = parent;
 
 	wxDropSource source(this);
@@ -871,8 +877,8 @@ void CRemoteTreeView::OnBeginDrag(wxTreeEvent& event)
 #if FZ3_USESHELLEXT
 	if (ext) {
 		if (!pRemoteDataObject->DidSendData()) {
-			server = m_state.GetServer();
-			if (!server || !m_state.IsRemoteIdle() || server != server) {
+			Site newSite = m_state.GetSite();
+			if (!newSite || !m_state.IsRemoteIdle() || site != newSite) {
 				wxBell();
 				return;
 			}
@@ -898,20 +904,26 @@ void CRemoteTreeView::OnContextMenu(wxTreeEvent& event)
 	auto item = new wxMenuItem(&menu, XRCID("ID_DOWNLOAD"), _("&Download"), _("Download selected directory"));
 	item->SetBitmap(wxArtProvider::GetBitmap(_T("ART_DOWNLOAD"), wxART_MENU));
 	menu.Append(item);
-    item = new wxMenuItem(&menu, XRCID("ID_ADDTOQUEUE"), _("&Add to queue"), _("Add selected directory to the transfer queue"));
+	item = new wxMenuItem(&menu, XRCID("ID_ADDTOQUEUE"), _("&Add to queue"), _("Add selected directory to the transfer queue"));
 	item->SetBitmap(wxArtProvider::GetBitmap(_T("ART_DOWNLOADADD"), wxART_MENU));
 	menu.Append(item);
 
-    menu.AppendSeparator();
-    menu.Append(XRCID("ID_MKDIR"), _("&Create directory"), _("Create a new subdirectory in the current directory"));
-    menu.Append(XRCID("ID_MKDIR_CHGDIR"), _("Create director&y and enter it"), _("Create a new subdirectory in the current directory and change into it"));
+	menu.AppendSeparator();
+	menu.Append(XRCID("ID_MKDIR"), _("&Create directory"), _("Create a new subdirectory in the current directory"));
+	menu.Append(XRCID("ID_MKDIR_CHGDIR"), _("Create director&y and enter it"), _("Create a new subdirectory in the current directory and change into it"));
 
-    menu.AppendSeparator();
-    menu.Append(XRCID("ID_DELETE"), _("D&elete"), _("Delete selected directory"));
-    menu.Append(XRCID("ID_RENAME"), _("&Rename"), _("Rename selected directory"));
-    menu.Append(XRCID("ID_GETURL"), _("C&opy URL(s) to clipboard"), _("Copy the URLs of the selected items to clipboard."));
-    menu.Append(XRCID("ID_GETURL_PASSWORD"), _("C&opy URL(s) with password to clipboard"), _("Copy the URLs of the selected items to clipboard, including password."));
-    menu.Append(XRCID("ID_CHMOD"), _("&File Attributes..."), _("Change the file permissions."));
+	menu.AppendSeparator();
+	menu.Append(XRCID("ID_DELETE"), _("D&elete"), _("Delete selected directory"));
+	menu.Append(XRCID("ID_RENAME"), _("&Rename"), _("Rename selected directory"));
+	menu.Append(XRCID("ID_GETURL"), _("C&opy URL(s) to clipboard"), _("Copy the URLs of the selected items to clipboard."));
+	menu.Append(XRCID("ID_GETURL_PASSWORD"), _("C&opy URL(s) with password to clipboard"), _("Copy the URLs of the selected items to clipboard, including password."));
+
+	auto const protocol = m_state.GetSite().server.GetProtocol();
+	bool const hasChmod = protocol == FTP || protocol == FTPS || protocol == FTPES || protocol == INSECURE_FTP || protocol == SFTP;
+
+	if (hasChmod) {
+	    menu.Append(XRCID("ID_CHMOD"), _("&File Attributes..."), _("Change the file permissions."));
+	}
 
 	const CServerPath& path = m_contextMenuItem ? GetPathFromItem(m_contextMenuItem) : CServerPath();
 	if (!m_state.IsRemoteIdle() || path.empty()) {
@@ -920,7 +932,9 @@ void CRemoteTreeView::OnContextMenu(wxTreeEvent& event)
 		menu.Enable(XRCID("ID_MKDIR"), false);
 		menu.Enable(XRCID("ID_MKDIR_CHGDIR"), false);
 		menu.Enable(XRCID("ID_DELETE"), false);
-		menu.Enable(XRCID("ID_CHMOD"), false);
+		if (hasChmod) {
+			menu.Enable(XRCID("ID_CHMOD"), false);
+		}
 		menu.Enable(XRCID("ID_RENAME"), false);
 		menu.Enable(XRCID("ID_GETURL"), false);
 	}
@@ -970,12 +984,13 @@ void CRemoteTreeView::OnMenuChmod(wxCommandEvent&)
 		// ... and it needs to be cached
 		cached = m_state.m_pEngine->CacheLookup(parentPath, listing) == FZ_REPLY_OK;
 		if (cached) {
-			for (unsigned int i = 0; i < listing.GetCount(); ++i) {
-				if (listing[i].name != name) {
+			for (size_t i = 0; i < listing.size(); ++i) {
+				auto const& entry = listing[i];
+				if (entry.name != name) {
 					continue;
 				}
 
-				pChmodDlg->ConvertPermissions(*listing[i].permissions, permissions);
+				pChmodDlg->ConvertPermissions(*entry.permissions, permissions);
 			}
 		}
 	}
@@ -1109,7 +1124,7 @@ void CRemoteTreeView::OnMenuDelete(wxCommandEvent&)
 	bool const hasParent = pathToDelete.HasParent();
 
 	CFilterManager filter;
-	if (CServer::ProtocolHasFeature(m_state.GetServer().server.GetProtocol(), ProtocolFeature::RecursiveDelete) && !filter.HasActiveRemoteFilters()) {
+	if (CServer::ProtocolHasFeature(m_state.GetSite().server.GetProtocol(), ProtocolFeature::RecursiveDelete) && !filter.HasActiveRemoteFilters()) {
 		if (hasParent) {
 			std::wstring const name = GetItemText(m_contextMenuItem).ToStdWstring();
 			m_state.m_pCommandQueue->ProcessCommand(new CRemoveDirCommand(pathToDelete.GetParent(), name));
@@ -1451,8 +1466,8 @@ void CRemoteTreeView::OnMenuGeturl(wxCommandEvent& event)
 		return;
 	}
 
-	ServerWithCredentials const& server = m_state.GetServer();
-	if (!server) {
+	Site const& site = m_state.GetSite();
+	if (!site) {
 		wxBell();
 		return;
 	}
@@ -1462,7 +1477,7 @@ void CRemoteTreeView::OnMenuGeturl(wxCommandEvent& event)
 		return;
 	}
 
-	wxString url = server.Format((event.GetId() == XRCID("ID_GETURL_PASSWORD")) ? ServerFormat::url_with_password : ServerFormat::url);
+	std::wstring url = site.Format((event.GetId() == XRCID("ID_GETURL_PASSWORD")) ? ServerFormat::url_with_password : ServerFormat::url);
 
 	std::wstring const pathPart = fz::percent_encode_w(path.GetPath(), true);
 	if (!pathPart.empty() && pathPart[0] != '/') {
