@@ -26,10 +26,10 @@ EVT_TREE_BEGIN_LABEL_EDIT(XRCID("ID_TREE"), CBookmarksDialog::OnBeginLabelEdit)
 EVT_TREE_END_LABEL_EDIT(XRCID("ID_TREE"), CBookmarksDialog::OnEndLabelEdit)
 END_EVENT_TABLE()
 
-CNewBookmarkDialog::CNewBookmarkDialog(wxWindow* parent, std::wstring& site_path, ServerWithCredentials const* server)
+CNewBookmarkDialog::CNewBookmarkDialog(wxWindow* parent, std::wstring& site_path, Site const* site)
 	: m_parent(parent)
 	, m_site_path(site_path)
-	, m_server(server)
+	, site_(site)
 {
 }
 
@@ -44,7 +44,7 @@ int CNewBookmarkDialog::Run(const wxString &local_path, const CServerPath &remot
 		xrc_call(*this, "ID_REMOTEPATH", &wxTextCtrl::ChangeValue, remote_path.GetPath());
 	}
 
-	if (!m_server) {
+	if (!site_) {
 		xrc_call(*this, "ID_TYPE_SITE", &wxRadioButton::Enable, false);
 	}
 
@@ -66,8 +66,8 @@ void CNewBookmarkDialog::OnOK(wxCommandEvent&)
 
 	CServerPath remote_path;
 	if (!remote_path_raw.empty()) {
-		if (!global && m_server) {
-			remote_path.SetType(m_server->server.GetType());
+		if (!global && site_) {
+			remote_path.SetType(site_->server.GetType());
 		}
 		if (!remote_path.SetPath(remote_path_raw.ToStdWstring())) {
 			wxMessageBoxEx(_("Could not parse remote path."), _("New bookmark"), wxICON_EXCLAMATION);
@@ -88,7 +88,7 @@ void CNewBookmarkDialog::OnOK(wxCommandEvent&)
 
 	bool const comparison = xrc_call(*this, "ID_COMPARISON", &wxCheckBox::GetValue);
 
-	if (!global && m_server) {
+	if (!global && site_) {
 		std::unique_ptr<Site> site;
 		if (!m_site_path.empty()) {
 			site = CSiteManager::GetSiteByPath(m_site_path).first;
@@ -98,7 +98,7 @@ void CNewBookmarkDialog::OnOK(wxCommandEvent&)
 				return;
 			}
 
-			m_site_path = CSiteManager::AddServer(*m_server);
+			m_site_path = CSiteManager::AddServer(*site);
 			if (m_site_path.empty()) {
 				wxMessageBoxEx(_("Could not add connection to Site Manager"), _("New bookmark"), wxICON_EXCLAMATION, this);
 				EndModal(wxID_CANCEL);
@@ -144,7 +144,7 @@ public:
 	{
 	}
 
-	CBookmarkItemData(const wxString& local_dir, const CServerPath& remote_dir, bool sync, bool comparison)
+	CBookmarkItemData(std::wstring const& local_dir, const CServerPath& remote_dir, bool sync, bool comparison)
 		: m_local_dir(local_dir), m_remote_dir(remote_dir), m_sync(sync)
 		, m_comparison(comparison)
 	{
@@ -156,10 +156,10 @@ public:
 	bool m_comparison{};
 };
 
-CBookmarksDialog::CBookmarksDialog(wxWindow* parent, std::wstring& site_path, ServerWithCredentials const* server)
+CBookmarksDialog::CBookmarksDialog(wxWindow* parent, std::wstring& site_path, Site const* site)
 	: m_parent(parent)
 	, m_site_path(site_path)
-	, m_server(server)
+	, site_(site)
 	, m_pTree()
 {
 }
@@ -178,7 +178,7 @@ void CBookmarksDialog::LoadGlobalBookmarks()
 
 	for (auto bookmark = element.child("Bookmark"); bookmark; bookmark = bookmark.next_sibling("Bookmark")) {
 		wxString name;
-		wxString local_dir;
+		std::wstring local_dir;
 		std::wstring remote_dir_raw;
 		CServerPath remote_dir;
 
@@ -258,7 +258,7 @@ int CBookmarksDialog::Run()
 	m_bookmarks_global = m_pTree->AppendItem(root, _("Global bookmarks"), 0, 0);
 	LoadGlobalBookmarks();
 	m_pTree->Expand(m_bookmarks_global);
-	if (m_server) {
+	if (site_) {
 		m_bookmarks_site = m_pTree->AppendItem(root, _("Site-specific bookmarks"), 0, 0);
 		LoadSiteSpecificBookmarks();
 		m_pTree->Expand(m_bookmarks_site);
@@ -420,26 +420,26 @@ bool CBookmarksDialog::Verify()
 		return true;
 	}
 
-	ServerWithCredentials const* server;
+	Site const* site;
 	if (m_pTree->GetItemParent(item) == m_bookmarks_site) {
-		server = m_server;
+		site = site_;
 	}
 	else {
-		server = 0;
+		site = 0;
 	}
 
 	wxString const remotePathRaw = xrc_call(*this, "ID_BOOKMARK_REMOTEDIR", &wxTextCtrl::GetValue);
 	if (!remotePathRaw.empty()) {
 		CServerPath remotePath;
-		if (server) {
-			remotePath.SetType(server->server.GetType());
+		if (site) {
+			remotePath.SetType(site->server.GetType());
 		}
 		if (!remotePath.SetPath(remotePathRaw.ToStdWstring())) {
 			xrc_call(*this, "ID_BOOKMARK_REMOTEDIR", &wxTextCtrl::SetFocus);
-			if (server) {
+			if (site) {
 				wxString msg;
-				if (server->server.GetType() != DEFAULT) {
-					msg = wxString::Format(_("Remote path cannot be parsed. Make sure it is a valid absolute path and is supported by the current site's servertype (%s)."), CServer::GetNameFromServerType(server->server.GetType()));
+				if (site->server.GetType() != DEFAULT) {
+					msg = wxString::Format(_("Remote path cannot be parsed. Make sure it is a valid absolute path and is supported by the current site's servertype (%s)."), CServer::GetNameFromServerType(site->server.GetType()));
 				}
 				else {
 					msg = _("Remote path cannot be parsed. Make sure it is a valid absolute path.");
@@ -482,23 +482,23 @@ void CBookmarksDialog::UpdateBookmark()
 		return;
 	}
 
-	ServerWithCredentials const* server;
+	Site const* site;
 	if (m_pTree->GetItemParent(item) == m_bookmarks_site) {
-		server = m_server;
+		site = site_;
 	}
 	else {
-		server = 0;
+		site = 0;
 	}
 
 	wxString const remotePathRaw = xrc_call(*this, "ID_BOOKMARK_REMOTEDIR", &wxTextCtrl::GetValue);
 	if (!remotePathRaw.empty()) {
-		if (server) {
-			data->m_remote_dir.SetType(server->server.GetType());
+		if (site) {
+			data->m_remote_dir.SetType(site->server.GetType());
 		}
 		data->m_remote_dir.SetPath(remotePathRaw.ToStdWstring());
 	}
 
-	data->m_local_dir = xrc_call(*this, "ID_BOOKMARK_LOCALDIR", &wxTextCtrl::GetValue);
+	data->m_local_dir = xrc_call(*this, "ID_BOOKMARK_LOCALDIR", &wxTextCtrl::GetValue).ToStdWstring();
 
 	data->m_sync = xrc_call(*this, "ID_BOOKMARK_SYNC", &wxCheckBox::GetValue);
 	data->m_comparison = xrc_call(*this, "ID_BOOKMARK_COMPARISON", &wxCheckBox::GetValue);
@@ -508,8 +508,8 @@ void CBookmarksDialog::DisplayBookmark()
 {
 	wxTreeItemId item = m_pTree->GetSelection();
 	if (!item) {
-		xrc_call(*this, "ID_BOOKMARK_REMOTEDIR", &wxTextCtrl::ChangeValue, _T(""));
-		xrc_call(*this, "ID_BOOKMARK_LOCALDIR", &wxTextCtrl::ChangeValue, _T(""));
+		xrc_call(*this, "ID_BOOKMARK_REMOTEDIR", &wxTextCtrl::ChangeValue, L"");
+		xrc_call(*this, "ID_BOOKMARK_LOCALDIR", &wxTextCtrl::ChangeValue, L"");
 		xrc_call(*this, "ID_DELETE", &wxButton::Enable, false);
 		xrc_call(*this, "ID_RENAME", &wxButton::Enable, false);
 		xrc_call(*this, "ID_COPY", &wxButton::Enable, false);
@@ -519,8 +519,8 @@ void CBookmarksDialog::DisplayBookmark()
 
 	CBookmarkItemData *data = (CBookmarkItemData *)m_pTree->GetItemData(item);
 	if (!data) {
-		xrc_call(*this, "ID_BOOKMARK_REMOTEDIR", &wxTextCtrl::ChangeValue, _T(""));
-		xrc_call(*this, "ID_BOOKMARK_LOCALDIR", &wxTextCtrl::ChangeValue, _T(""));
+		xrc_call(*this, "ID_BOOKMARK_REMOTEDIR", &wxTextCtrl::ChangeValue, L"");
+		xrc_call(*this, "ID_BOOKMARK_LOCALDIR", &wxTextCtrl::ChangeValue, L"");
 		xrc_call(*this, "ID_DELETE", &wxButton::Enable, false);
 		xrc_call(*this, "ID_RENAME", &wxButton::Enable, false);
 		xrc_call(*this, "ID_COPY", &wxButton::Enable, false);
@@ -568,7 +568,7 @@ void CBookmarksDialog::OnNewBookmark(wxCommandEvent&)
 				return;
 			}
 
-			m_site_path = CSiteManager::AddServer(*m_server);
+			m_site_path = CSiteManager::AddServer(*site_);
 			if (m_site_path.empty()) {
 				wxMessageBoxEx(_("Could not add connection to Site Manager"), _("New bookmark"), wxICON_EXCLAMATION, this);
 				return;
@@ -738,7 +738,7 @@ void CBookmarksDialog::OnEndLabelEdit(wxTreeEvent& event)
 	m_pTree->SortChildren(parent);
 }
 
-bool CBookmarksDialog::GetGlobalBookmarks(std::vector<wxString> &bookmarks)
+bool CBookmarksDialog::GetGlobalBookmarks(std::vector<std::wstring> &bookmarks)
 {
 	CInterProcessMutex mutex(MUTEX_GLOBALBOOKMARKS);
 
@@ -751,8 +751,8 @@ bool CBookmarksDialog::GetGlobalBookmarks(std::vector<wxString> &bookmarks)
 	}
 
 	for (auto bookmark = element.child("Bookmark"); bookmark; bookmark = bookmark.next_sibling("Bookmark")) {
-		wxString name;
-		wxString local_dir;
+		std::wstring name;
+		std::wstring local_dir;
 		std::wstring remote_dir_raw;
 		CServerPath remote_dir;
 
