@@ -21,6 +21,10 @@
 #include "viewheader.h"
 #include "xmlfunctions.h"
 
+#ifdef USE_MAC_SANDBOX
+#include "osx_sandbox_userdirs.h"
+#endif
+
 #include <wx/wupdlock.h>
 
 DECLARE_EVENT_TYPE(fzEVT_TAB_CLOSING_DEFERRED, -1)
@@ -114,10 +118,17 @@ void CContextControl::CreateTab(CLocalPath const& localPath, Site const& site, C
 		pState->GetRemoteRecursiveOperation()->SetQueue(m_mainFrame.GetQueue());
 
 		if (localPath.empty() || !pState->SetLocalDir(localPath)) {
+#ifdef USE_MAC_SANDBOX
+			auto const dirs = OSXSandboxUserdirs::Get().GetDirs();
+			if (dirs.empty() || !pState->SetLocalDir(dirs.front())) {
+				pState->SetLocalDir(L"/");
+			}
+#else
 			std::wstring const homeDir = wxGetHomeDir().ToStdWstring();
 			if (!pState->SetLocalDir(homeDir)) {
-				pState->SetLocalDir(_T("/"));
+				pState->SetLocalDir(L"/");
 			}
+#endif
 		}
 
 		CContextManager::Get()->SetCurrentContext(pState);
@@ -213,7 +224,9 @@ void CContextControl::CreateContextControls(CState& state)
 		pLocalFilelistStatusBar->Hide();
 	}
 	context_controls.pLocalListViewPanel->SetStatusBar(pLocalFilelistStatusBar);
-	context_controls.pLocalListView->SetFilelistStatusBar(pLocalFilelistStatusBar);
+	if (context_controls.pLocalListView) {
+		context_controls.pLocalListView->SetFilelistStatusBar(pLocalFilelistStatusBar);
+	}
 	pLocalFilelistStatusBar->SetConnected(true);
 
 	CFilelistStatusBar* pRemoteFilelistStatusBar = new CFilelistStatusBar(context_controls.pRemoteListViewPanel);
@@ -221,7 +234,9 @@ void CContextControl::CreateContextControls(CState& state)
 		pRemoteFilelistStatusBar->Hide();
 	}
 	context_controls.pRemoteListViewPanel->SetStatusBar(pRemoteFilelistStatusBar);
-	context_controls.pRemoteListView->SetFilelistStatusBar(pRemoteFilelistStatusBar);
+	if (context_controls.pRemoteListView) {
+		context_controls.pRemoteListView->SetFilelistStatusBar(pRemoteFilelistStatusBar);
+	}
 
 	auto localRecursiveStatus = new CRecursiveOperationStatus(context_controls.pLocalListViewPanel, state, true);
 	context_controls.pLocalListViewPanel->SetFooter(localRecursiveStatus);
@@ -311,7 +326,9 @@ void CContextControl::CreateContextControls(CState& state)
 	m_mainFrame.ConnectNavigationHandler(context_controls.pLocalViewHeader);
 	m_mainFrame.ConnectNavigationHandler(context_controls.pRemoteViewHeader);
 
-	state.GetComparisonManager()->SetListings(context_controls.pLocalListView, context_controls.pRemoteListView);
+	if (context_controls.pLocalListView && context_controls.pRemoteListView) {
+		state.GetComparisonManager()->SetListings(context_controls.pLocalListView, context_controls.pRemoteListView);
+	}
 
 	if (m_tabs) {
 		m_tabs->AddPage(context_controls.pViewSplitter, state.GetTitle());
@@ -581,7 +598,7 @@ void CContextControl::AdvanceTab(bool forward)
 	m_tabs->AdvanceTab(forward);
 }
 
-void CContextControl::OnStateChange(CState* pState, t_statechange_notifications notification, const wxString&, const void*)
+void CContextControl::OnStateChange(CState* pState, t_statechange_notifications notification, std::wstring const&, const void*)
 {
 	if (notification == STATECHANGE_CHANGEDCONTEXT) {
 		if (!pState) {
@@ -639,8 +656,8 @@ void CContextControl::SaveTabs()
 		Site const site = controls->pState->GetLastSite();
 
 		auto tab = tabs.append_child("Tab");
-		SetServer(tab, site.server_);
-		tab.append_child("Site").text().set(fz::to_utf8(site.m_path).c_str());
+		SetServer(tab, site);
+		tab.append_child("Site").text().set(fz::to_utf8(site.SitePath()).c_str());
 		tab.append_child("RemotePath").text().set(fz::to_utf8(controls->pState->GetLastServerPath().GetSafePath()).c_str());
 		tab.append_child("LocalPath").text().set(fz::to_utf8(controls->pState->GetLocalDir().GetPath()).c_str());
 
@@ -686,17 +703,15 @@ void CContextControl::RestoreTabs()
 			CLocalPath localPath(fz::to_wstring_from_utf8(tab.child("LocalPath").child_value()));
 
 			Site site;
-
-			ServerWithCredentials last_server;
 			CServerPath last_path;
 
-			if (GetServer(tab, site.server_) && last_path.SetSafePath(fz::to_wstring_from_utf8(tab.child("RemotePath").child_value()))) {
+			if (GetServer(tab, site) && last_path.SetSafePath(fz::to_wstring_from_utf8(tab.child("RemotePath").child_value()))) {
 				std::wstring last_site_path = fz::to_wstring_from_utf8(tab.child("Site").child_value());
 
 				std::unique_ptr<Site> ssite;
 				if (!last_site_path.empty()) {
 					auto ssite = CSiteManager::GetSiteByPath(last_site_path, false).first;
-					if (ssite && ssite->server_ == site.server_) {
+					if (ssite && ssite->SameResource(site)) {
 						site = *ssite;
 					}
 				}
