@@ -21,7 +21,7 @@ class CListSearchPanel;
 class CContextControl final : public wxSplitterWindow, public CGlobalStateEventHandler
 {
 public:
-	struct _context_controls
+	struct _context_controls final
 	{
 		bool used() const { return pViewSplitter != 0; }
 
@@ -52,6 +52,7 @@ public:
 	void Create(wxWindow* parent);
 
 	void CreateTab();
+	void CreateTab(CLocalPath const& localPath, Site const& site, CServerPath const& remotePath);
 	bool CloseTab(int tab);
 
 	_context_controls* GetCurrentControls();
@@ -64,6 +65,9 @@ public:
 	bool SelectTab(int i);
 	void AdvanceTab(bool forward);
 
+	void SaveTabs();
+	void RestoreTabs();
+
 protected:
 
 	void CreateContextControls(CState& state);
@@ -87,7 +91,7 @@ protected:
 	void OnTabContextCloseOthers(wxCommandEvent& event);
 	void OnTabContextNew(wxCommandEvent&);
 
-	virtual void OnStateChange(CState* pState, t_statechange_notifications notification, const wxString&, const void*);
+	virtual void OnStateChange(CState* pState, t_statechange_notifications notification, std::wstring const&, const void*) override;
 };
 
 #endif
