@@ -1,5 +1,6 @@
 #include <filezilla.h>
 #include "fileexistsdlg.h"
+#include "file_utils.h"
 #include "Options.h"
 #include "sizeformatting.h"
 #include "timeformatting.h"
@@ -35,10 +36,9 @@ bool CFileExistsDlg::Create(wxWindow* parent)
 	return true;
 }
 
-void CFileExistsDlg::DisplayFile(bool left, wxString name, int64_t size, fz::datetime const& time, wxString const& iconFile)
+void CFileExistsDlg::DisplayFile(bool left, std::wstring const& name, int64_t size, fz::datetime const& time, wxString const& iconFile)
 {
-	name = GetPathEllipsis(name, FindWindow(left ? XRCID("ID_FILE1_NAME") : XRCID("ID_FILE2_NAME")));
-	name.Replace(_T("&"), _T("&&"));
+	std::wstring labelName = LabelEscape(GetPathEllipsis(name, FindWindow(left ? XRCID("ID_FILE1_NAME") : XRCID("ID_FILE2_NAME"))));
 
 	wxString sizeStr = _("Size unknown");
 	if (size >= 0) {
@@ -47,10 +47,11 @@ void CFileExistsDlg::DisplayFile(bool left, wxString name, int64_t size, fz::dat
 	}
 
 	wxString timeStr = _("Date/time unknown");
-	if (!time.empty())
+	if (!time.empty()) {
 		timeStr = CTimeFormat::Format(time);
+	}
 
-	xrc_call(*this, left ? "ID_FILE1_NAME" : "ID_FILE2_NAME", &wxStaticText::SetLabel, name);
+	xrc_call(*this, left ? "ID_FILE1_NAME" : "ID_FILE2_NAME", &wxStaticText::SetLabel, labelName);
 	xrc_call(*this, left ? "ID_FILE1_SIZE" : "ID_FILE2_SIZE", &wxStaticText::SetLabel, sizeStr);
 	xrc_call(*this, left ? "ID_FILE1_TIME" : "ID_FILE2_TIME", &wxStaticText::SetLabel, timeStr);
 
@@ -63,8 +64,8 @@ bool CFileExistsDlg::CreateControls()
 		return false;
 	}
 
-	wxString localFile = m_pNotification->localFile;
-	wxString remoteFile = m_pNotification->remotePath.FormatFilename(m_pNotification->remoteFile);
+	std::wstring const& localFile = m_pNotification->localFile;
+	std::wstring remoteFile = m_pNotification->remotePath.FormatFilename(m_pNotification->remoteFile);
 
 	DisplayFile(m_pNotification->download, localFile, m_pNotification->localSize, m_pNotification->localTime, m_pNotification->localFile);
 	DisplayFile(!m_pNotification->download, remoteFile, m_pNotification->remoteSize, m_pNotification->remoteTime, m_pNotification->remoteFile);
@@ -77,13 +78,14 @@ bool CFileExistsDlg::CreateControls()
 void CFileExistsDlg::LoadIcon(int id, const wxString &file)
 {
 	wxStaticBitmap *pStatBmp = static_cast<wxStaticBitmap *>(FindWindow(id));
-	if (!pStatBmp)
+	if (!pStatBmp) {
 		return;
+	}
 
 	wxSize size = CThemeProvider::GetIconSize(iconSizeNormal);
 	pStatBmp->SetInitialSize(size);
 	pStatBmp->InvalidateBestSize();
-	pStatBmp->SetBitmap(CThemeProvider::GetBitmap(_T("ART_FILE"), wxART_OTHER, size));
+	pStatBmp->SetBitmap(CThemeProvider::Get()->CreateBitmap(_T("ART_FILE"), wxART_OTHER, size));
 
 #ifdef __WXMSW__
 	SHFILEINFO fileinfo;
@@ -117,10 +119,10 @@ void CFileExistsDlg::LoadIcon(int id, const wxString &file)
 
 #endif //__WXMSW__
 
-	wxFileName fn(file);
-	wxString ext = fn.GetExt();
-	if (ext.empty())
+	std::wstring ext = GetExtension(file.ToStdWstring());
+	if (ext.empty()) {
 		return;
+	}
 
 	wxFileType *pType = wxTheMimeTypesManager->GetFileTypeFromExtension(ext);
 	if (pType) {
@@ -165,22 +167,30 @@ void CFileExistsDlg::LoadIcon(int id, const wxString &file)
 
 void CFileExistsDlg::OnOK(wxCommandEvent&)
 {
-	if (xrc_call(*this, "ID_ACTION1", &wxRadioButton::GetValue))
+	if (xrc_call(*this, "ID_ACTION1", &wxRadioButton::GetValue)) {
 		m_action = CFileExistsNotification::overwrite;
-	else if (xrc_call(*this, "ID_ACTION2", &wxRadioButton::GetValue))
+	}
+	else if (xrc_call(*this, "ID_ACTION2", &wxRadioButton::GetValue)) {
 		m_action = CFileExistsNotification::overwriteNewer;
-	else if (xrc_call(*this, "ID_ACTION3", &wxRadioButton::GetValue))
+	}
+	else if (xrc_call(*this, "ID_ACTION3", &wxRadioButton::GetValue)) {
 		m_action = CFileExistsNotification::resume;
-	else if (xrc_call(*this, "ID_ACTION4", &wxRadioButton::GetValue))
+	}
+	else if (xrc_call(*this, "ID_ACTION4", &wxRadioButton::GetValue)) {
 		m_action = CFileExistsNotification::rename;
-	else if (xrc_call(*this, "ID_ACTION5", &wxRadioButton::GetValue))
+	}
+	else if (xrc_call(*this, "ID_ACTION5", &wxRadioButton::GetValue)) {
 		m_action = CFileExistsNotification::skip;
-	else if (xrc_call(*this, "ID_ACTION6", &wxRadioButton::GetValue))
+	}
+	else if (xrc_call(*this, "ID_ACTION6", &wxRadioButton::GetValue)) {
 		m_action = CFileExistsNotification::overwriteSizeOrNewer;
-	else if (xrc_call(*this, "ID_ACTION7", &wxRadioButton::GetValue))
+	}
+	else if (xrc_call(*this, "ID_ACTION7", &wxRadioButton::GetValue)) {
 		m_action = CFileExistsNotification::overwriteSize;
-	else
+	}
+	else {
 		m_action = CFileExistsNotification::overwrite;
+	}
 
 	m_always = xrc_call(*this, "ID_ALWAYS", &wxCheckBox::GetValue);
 	m_directionOnly = xrc_call(*this, "ID_UPDOWNONLY", &wxCheckBox::GetValue);
@@ -206,7 +216,7 @@ bool CFileExistsDlg::Always(bool &directionOnly, bool &queueOnly) const
 	return m_always;
 }
 
-wxString CFileExistsDlg::GetPathEllipsis(wxString path, wxWindow *window)
+std::wstring CFileExistsDlg::GetPathEllipsis(std::wstring const& path, wxWindow *window)
 {
 	int dn = wxDisplay::GetFromWindow(GetParent()); // Use parent window as the dialog isn't realized yet.
 	if (dn < 0) {
@@ -223,8 +233,9 @@ wxString CFileExistsDlg::GetPathEllipsis(wxString path, wxWindow *window)
 	const int maxWidth = (int)(DESKTOP_WIDTH * 0.75);
 
 	// If the path is already short enough, don't change it
-	if (string_width <= maxWidth || path.Length() < 20)
+	if (string_width <= maxWidth || path.size() < 20) {
 		return path;
+	}
 
 	wxString fill = _T(" ");
 	fill += 0x2026; //unicode ellipsis character
@@ -234,9 +245,9 @@ wxString CFileExistsDlg::GetPathEllipsis(wxString path, wxWindow *window)
 	window->GetTextExtent(fill, &fillWidth, &y);
 
 	// Do initial split roughly in the middle of the string
-	int middle = path.Length() / 2;
-	wxString left = path.Left(middle);
-	wxString right = path.Mid(middle);
+	size_t middle = path.size() / 2;
+	wxString left = path.substr(0, middle);
+	wxString right = path.substr(middle);
 
 	int leftWidth, rightWidth;
 	window->GetTextExtent(left, &leftWidth, &y);
@@ -249,22 +260,25 @@ wxString CFileExistsDlg::GetPathEllipsis(wxString path, wxWindow *window)
 			window->GetTextExtent(left, &leftWidth, &y);
 		}
 		else {
-			if (right.Len() <= 10)
+			if (right.Len() <= 10) {
 				break;
+			}
 
 			right = right.Mid(1);
 			window->GetTextExtent(right, &rightWidth, &y);
 		}
 	}
 
-	return left + fill + right;
+	return (left + fill + right).ToStdWstring();
 }
 
 void CFileExistsDlg::OnCheck(wxCommandEvent& event)
 {
-	if (event.GetId() != XRCID("ID_UPDOWNONLY") && event.GetId() != XRCID("ID_QUEUEONLY"))
+	if (event.GetId() != XRCID("ID_UPDOWNONLY") && event.GetId() != XRCID("ID_QUEUEONLY")) {
 		return;
+	}
 
-	if (event.IsChecked())
+	if (event.IsChecked()) {
 		XRCCTRL(*this, "ID_ALWAYS", wxCheckBox)->SetValue(true);
+	}
 }
