@@ -6,6 +6,9 @@
 #include "Options.h"
 #include "queue.h"
 
+#include <libfilezilla/local_filesys.hpp>
+#include <libfilezilla/recursive_remove.hpp>
+
 recursion_root::recursion_root(CServerPath const& start_dir, bool allow_parent)
 	: m_remoteStartDir(start_dir)
 	, m_allowParent(allow_parent)
@@ -13,7 +16,7 @@ recursion_root::recursion_root(CServerPath const& start_dir, bool allow_parent)
 	wxASSERT_MSG(!start_dir.empty(), _T("Empty startDir in recursion_root constructor"));
 }
 
-void recursion_root::add_dir_to_visit(CServerPath const& path, wxString const& subdir, CLocalPath const& localDir, bool is_link)
+void recursion_root::add_dir_to_visit(CServerPath const& path, std::wstring const& subdir, CLocalPath const& localDir, bool is_link)
 {
 	new_dir dirToVisit;
 
@@ -24,13 +27,13 @@ void recursion_root::add_dir_to_visit(CServerPath const& path, wxString const& s
 	m_dirsToVisit.push_back(dirToVisit);
 }
 
-void recursion_root::add_dir_to_visit_restricted(CServerPath const& path, wxString const& restrict, bool recurse)
+void recursion_root::add_dir_to_visit_restricted(CServerPath const& path, std::wstring const& restrict, bool recurse)
 {
 	new_dir dirToVisit;
 	dirToVisit.parent = path;
 	dirToVisit.recurse = recurse;
 	if (!restrict.empty()) {
-		dirToVisit.restrict = fz::sparse_optional<wxString>(restrict);
+		dirToVisit.restrict = fz::sparse_optional<std::wstring>(restrict);
 	}
 	m_dirsToVisit.push_back(dirToVisit);
 }
@@ -50,7 +53,7 @@ CRemoteRecursiveOperation::~CRemoteRecursiveOperation()
 	}
 }
 
-void CRemoteRecursiveOperation::OnStateChange(t_statechange_notifications notification, const wxString&, const void* data2)
+void CRemoteRecursiveOperation::OnStateChange(t_statechange_notifications notification, std::wstring const&, const void* data2)
 {
 	if (notification == STATECHANGE_REMOTE_DIR_OTHER && data2) {
 		std::shared_ptr<CDirectoryListing> const& listing = *reinterpret_cast<std::shared_ptr<CDirectoryListing> const*>(data2);
@@ -69,17 +72,19 @@ void CRemoteRecursiveOperation::AddRecursionRoot(recursion_root && root)
 	}
 }
 
-void CRemoteRecursiveOperation::StartRecursiveOperation(OperationMode mode, std::vector<CFilter> const& filters, CServerPath const& finalDir)
+void CRemoteRecursiveOperation::StartRecursiveOperation(OperationMode mode, ActiveFilters const& filters, CServerPath const& finalDir, bool immediate)
 {
 	wxCHECK_RET(m_operationMode == recursive_none, _T("StartRecursiveOperation called with m_operationMode != recursive_none"));
 	wxCHECK_RET(m_state.IsRemoteConnected(), _T("StartRecursiveOperation while disconnected"));
 	wxCHECK_RET(!finalDir.empty(), _T("Empty final dir in recursive operation"));
 
-	if (mode == recursive_chmod && !m_pChmodDlg)
+	if (mode == recursive_chmod && !m_pChmodDlg) {
 		return;
+	}
 
-	if ((mode == recursive_transfer || mode == recursive_addtoqueue || mode == recursive_transfer_flatten || mode == recursive_addtoqueue_flatten) && !m_pQueue)
+	if ((mode == recursive_transfer || mode == recursive_transfer_flatten) && !m_pQueue) {
 		return;
+	}
 
 	if (recursion_roots_.empty()) {
 		// Nothing to do in this case
@@ -89,7 +94,13 @@ void CRemoteRecursiveOperation::StartRecursiveOperation(OperationMode mode, std:
 	m_processedFiles = 0;
 	m_processedDirectories = 0;
 
+	m_immediate = immediate;
 	m_operationMode = mode;
+
+	if ((mode == CRecursiveOperation::recursive_transfer || mode == CRecursiveOperation::recursive_transfer_flatten) && immediate) {
+		m_actionAfterBlocker = m_pQueue->GetActionAfterBlocker();
+	}
+
 	m_state.NotifyHandlers(STATECHANGE_REMOTE_IDLE);
 	m_state.NotifyHandlers(STATECHANGE_REMOTE_RECURSION_STATUS);
 
@@ -100,8 +111,9 @@ void CRemoteRecursiveOperation::StartRecursiveOperation(OperationMode mode, std:
 
 bool CRemoteRecursiveOperation::NextOperation()
 {
-	if (m_operationMode == recursive_none)
+	if (m_operationMode == recursive_none) {
 		return false;
+	}
 
 	while (!recursion_roots_.empty()) {
 		auto & root = recursion_roots_.front();
@@ -126,7 +138,7 @@ bool CRemoteRecursiveOperation::NextOperation()
 		auto curPath = m_state.GetRemotePath();
 		if (!curPath.empty() && (curPath == m_finalDir || m_finalDir.IsParentOf(curPath, false))) {
 			StopRecursiveOperation();
-			m_state.ChangeRemoteDir(m_finalDir, wxString(), LIST_FLAG_REFRESH);
+			m_state.ChangeRemoteDir(m_finalDir, std::wstring(), LIST_FLAG_REFRESH);
 			return false;
 		}
 	}
@@ -139,20 +151,24 @@ bool CRemoteRecursiveOperation::NextOperation()
 bool CRemoteRecursiveOperation::BelowRecursionRoot(const CServerPath& path, recursion_root::new_dir &dir)
 {
 	if (!dir.start_dir.empty()) {
-		if (path.IsSubdirOf(dir.start_dir, false))
+		if (path.IsSubdirOf(dir.start_dir, false)) {
 			return true;
-		else
+		}
+		else {
 			return false;
+		}
 	}
 
 	auto & root = recursion_roots_.front();
-	if (path.IsSubdirOf(root.m_remoteStartDir, false))
+	if (path.IsSubdirOf(root.m_remoteStartDir, false)) {
 		return true;
+	}
 
 	// In some cases (chmod from tree for example) it is neccessary to list the
 	// parent first
-	if (path == root.m_remoteStartDir && root.m_allowParent)
+	if (path == root.m_remoteStartDir && root.m_allowParent) {
 		return true;
+	}
 
 	if (dir.link == 2) {
 		dir.start_dir = path;
@@ -163,7 +179,7 @@ bool CRemoteRecursiveOperation::BelowRecursionRoot(const CServerPath& path, recu
 }
 
 // Defined in RemoteListView.cpp
-extern wxString StripVMSRevision(const wxString& name);
+std::wstring StripVMSRevision(std::wstring const& name);
 
 void CRemoteRecursiveOperation::ProcessDirectoryListing(const CDirectoryListing* pDirectoryListing)
 {
@@ -172,8 +188,9 @@ void CRemoteRecursiveOperation::ProcessDirectoryListing(const CDirectoryListing*
 		return;
 	}
 
-	if (m_operationMode == recursive_none || recursion_roots_.empty())
+	if (m_operationMode == recursive_none || recursion_roots_.empty()) {
 		return;
+	}
 
 	if (pDirectoryListing->failed()) {
 		// Ignore this.
@@ -218,16 +235,19 @@ void CRemoteRecursiveOperation::ProcessDirectoryListing(const CDirectoryListing*
 
 	++m_processedDirectories;
 
-	const CServer* pServer = m_state.GetServer();
-	wxASSERT(pServer);
+	Site const& site = m_state.GetSite();
+	if (!site) {
+		StopRecursiveOperation();
+		return;
+	}
 
-	if (!pDirectoryListing->GetCount()) {
-		if (m_operationMode == recursive_transfer) {
+	if (!pDirectoryListing->size() && m_operationMode == recursive_transfer) {
+		if (m_immediate) {
 			wxFileName::Mkdir(dir.localDir.GetPath(), 0777, wxPATH_MKDIR_FULL);
 			m_state.RefreshLocalFile(dir.localDir.GetPath());
 		}
-		else if (m_operationMode == recursive_addtoqueue) {
-			m_pQueue->QueueFile(true, true, _T(""), _T(""), dir.localDir, CServerPath(), *pServer, -1);
+		else {
+			m_pQueue->QueueFile(true, true, _T(""), _T(""), dir.localDir, CServerPath(), site, -1);
 			m_pQueue->QueueFile_Finish(false);
 		}
 	}
@@ -237,21 +257,68 @@ void CRemoteRecursiveOperation::ProcessDirectoryListing(const CDirectoryListing*
 	// Is operation restricted to a single child?
 	bool const restrict = static_cast<bool>(dir.restrict);
 
-	std::deque<wxString> filesToDelete;
+	std::deque<std::wstring> filesToDelete;
 
-	const wxString path = pDirectoryListing->path.GetPath();
+	std::wstring const remotePath = pDirectoryListing->path.GetPath();
+
+	if (m_operationMode == recursive_synchronize_download && !dir.localDir.empty()) {
+		// Step one in synchronization: Delete local files not on the server
+		fz::local_filesys fs;
+		if (fs.begin_find_files(fz::to_native(dir.localDir.GetPath()))) {
+			std::list<fz::native_string> paths_to_delete;
+
+			bool isLink{};
+			fz::native_string name;
+			bool isDir{};
+			int64_t size{};
+			fz::datetime time;
+			int attributes{};
+			while (fs.get_next_file(name, isLink, isDir, &size, &time, &attributes)) {
+				if (isLink) {
+					continue;
+				}
+				auto const wname = fz::to_wstring(name);
+				if (filter.FilenameFiltered(m_filters.first, wname, dir.localDir.GetPath(), isDir, size, attributes, time)) {
+					continue;
+				}
+
+				// Local item isn't filtered
+
+				size_t remoteIndex = pDirectoryListing->FindFile_CmpCase(fz::to_wstring(name));
+				if (remoteIndex != std::string::npos) {
+					CDirentry const& entry = (*pDirectoryListing)[remoteIndex];
+					if (!filter.FilenameFiltered(m_filters.second, entry.name, remotePath, entry.is_dir(), entry.size, 0, entry.time)) {
+						// Both local and remote items exist
+
+						if (isDir == entry.is_dir() || entry.is_link()) {
+							// Normal item, nothing we should do
+							continue;
+						}
+					}
+				}
+
+				// Local item should be deleted if reaching this point
+				paths_to_delete.push_back(fz::to_native(dir.localDir.GetPath()) + name);
+			}
+
+			fz::recursive_remove r;
+			r.remove(paths_to_delete);
+		}
+	}
 
 	bool added = false;
 
-	for (int i = pDirectoryListing->GetCount() - 1; i >= 0; --i) {
-		const CDirentry& entry = (*pDirectoryListing)[i];
+	for (size_t i = pDirectoryListing->size(); i > 0; --i) {
+		const CDirentry& entry = (*pDirectoryListing)[i - 1];
 
 		if (restrict) {
-			if (entry.name != *dir.restrict)
+			if (entry.name != *dir.restrict) {
 				continue;
+			}
 		}
-		else if (filter.FilenameFiltered(m_filters, entry.name, path, entry.is_dir(), entry.size, 0, entry.time))
+		else if (filter.FilenameFiltered(m_filters.second, entry.name, remotePath, entry.is_dir(), entry.size, 0, entry.time)) {
 			continue;
+		}
 
 		if (!entry.is_dir()) {
 			++m_processedFiles;
@@ -265,7 +332,7 @@ void CRemoteRecursiveOperation::ProcessDirectoryListing(const CDirectoryListing*
 				dirToVisit.localDir = dir.localDir;
 				dirToVisit.start_dir = dir.start_dir;
 
-				if (m_operationMode == recursive_transfer || m_operationMode == recursive_addtoqueue) {
+				if (m_operationMode == recursive_transfer || m_operationMode == recursive_synchronize_download) {
 					// Non-flatten case
 					dirToVisit.localDir.AddSegment(CQueueView::ReplaceInvalidCharacters(entry.name));
 				}
@@ -281,25 +348,15 @@ void CRemoteRecursiveOperation::ProcessDirectoryListing(const CDirectoryListing*
 			{
 			case recursive_transfer:
 			case recursive_transfer_flatten:
+			case recursive_synchronize_download:
 				{
-					wxString localFile = CQueueView::ReplaceInvalidCharacters(entry.name);
-					if (pDirectoryListing->path.GetType() == VMS && COptions::Get()->GetOptionVal(OPTION_STRIP_VMS_REVISION))
+					std::wstring localFile = CQueueView::ReplaceInvalidCharacters(entry.name);
+					if (pDirectoryListing->path.GetType() == VMS && COptions::Get()->GetOptionVal(OPTION_STRIP_VMS_REVISION)) {
 						localFile = StripVMSRevision(localFile);
-					m_pQueue->QueueFile(m_operationMode == recursive_addtoqueue, true,
-						entry.name, (entry.name == localFile) ? wxString() : localFile,
-						dir.localDir, pDirectoryListing->path, *pServer, entry.size);
-					added = true;
-				}
-				break;
-			case recursive_addtoqueue:
-			case recursive_addtoqueue_flatten:
-				{
-					wxString localFile = CQueueView::ReplaceInvalidCharacters(entry.name);
-					if (pDirectoryListing->path.GetType() == VMS && COptions::Get()->GetOptionVal(OPTION_STRIP_VMS_REVISION))
-						localFile = StripVMSRevision(localFile);
-					m_pQueue->QueueFile(true, true,
-						entry.name, (entry.name == localFile) ? wxString() : localFile,
-						dir.localDir, pDirectoryListing->path, *pServer, entry.size);
+					}
+					m_pQueue->QueueFile(!m_immediate, true,
+						entry.name, (entry.name == localFile) ? std::wstring() : localFile,
+						dir.localDir, pDirectoryListing->path, site, entry.size);
 					added = true;
 				}
 				break;
@@ -319,16 +376,18 @@ void CRemoteRecursiveOperation::ProcessDirectoryListing(const CDirectoryListing*
 			{
 				char permissions[9];
 				bool res = m_pChmodDlg->ConvertPermissions(*entry.permissions, permissions);
-				wxString newPerms = m_pChmodDlg->GetPermissions(res ? permissions : 0, entry.is_dir());
+				std::wstring newPerms = m_pChmodDlg->GetPermissions(res ? permissions : 0, entry.is_dir()).ToStdWstring();
 				m_state.m_pCommandQueue->ProcessCommand(new CChmodCommand(pDirectoryListing->path, entry.name, newPerms), CCommandQueue::recursiveOperation);
 			}
 		}
 	}
-	if (added)
-		m_pQueue->QueueFile_Finish(m_operationMode != recursive_addtoqueue && m_operationMode != recursive_addtoqueue_flatten);
+	if (added) {
+		m_pQueue->QueueFile_Finish(m_immediate);
+	}
 
-	if (m_operationMode == recursive_delete && !filesToDelete.empty())
+	if (m_operationMode == recursive_delete && !filesToDelete.empty()) {
 		m_state.m_pCommandQueue->ProcessCommand(new CDeleteCommand(pDirectoryListing->path, std::move(filesToDelete)), CCommandQueue::recursiveOperation);
+	}
 
 	m_state.NotifyHandlers(STATECHANGE_REMOTE_RECURSION_STATUS);
 
@@ -358,14 +417,17 @@ void CRemoteRecursiveOperation::StopRecursiveOperation()
 		m_pChmodDlg->Destroy();
 		m_pChmodDlg = 0;
 	}
+
+	m_actionAfterBlocker.reset();
 }
 
 void CRemoteRecursiveOperation::ListingFailed(int error)
 {
-	if (m_operationMode == recursive_none || recursion_roots_.empty())
+	if (m_operationMode == recursive_none || recursion_roots_.empty()) {
 		return;
+	}
 
-	if( (error & FZ_REPLY_CANCELED) == FZ_REPLY_CANCELED) {
+	if ((error & FZ_REPLY_CANCELED) == FZ_REPLY_CANCELED) {
 		// User has cancelled operation
 		StopRecursiveOperation();
 		return;
@@ -382,14 +444,24 @@ void CRemoteRecursiveOperation::ListingFailed(int error)
 		dir.second_try = true;
 		root.m_dirsToVisit.push_front(dir);
 	}
+	else {
+		if (m_operationMode == recursive_delete && dir.doVisit && !dir.subdir.empty()) {
+			// After recursing into directory to delete its contents, delete directory itself
+			// Gets handled in NextOperation
+			recursion_root::new_dir dir2 = dir;
+			dir2.doVisit = false;
+			root.m_dirsToVisit.push_front(dir2);
+		}
+	}
 
 	NextOperation();
 }
 
 void CRemoteRecursiveOperation::LinkIsNotDir()
 {
-	if (m_operationMode == recursive_none || recursion_roots_.empty())
+	if (m_operationMode == recursive_none || recursion_roots_.empty()) {
 		return;
+	}
 
 	auto & root = recursion_roots_.front();
 	wxCHECK_RET(!root.m_dirsToVisit.empty(), _T("Empty dirs to visit"));
@@ -397,15 +469,15 @@ void CRemoteRecursiveOperation::LinkIsNotDir()
 	recursion_root::new_dir dir = root.m_dirsToVisit.front();
 	root.m_dirsToVisit.pop_front();
 
-	const CServer* pServer = m_state.GetServer();
-	if (!pServer) {
+	Site const& site = m_state.GetSite();
+	if (!site) {
 		NextOperation();
 		return;
 	}
 
 	if (m_operationMode == recursive_delete) {
 		if (!dir.subdir.empty()) {
-			std::deque<wxString> files;
+			std::deque<std::wstring> files;
 			files.push_back(dir.subdir);
 			m_state.m_pCommandQueue->ProcessCommand(new CDeleteCommand(dir.parent, std::move(files)), CCommandQueue::recursiveOperation);
 		}
@@ -414,11 +486,12 @@ void CRemoteRecursiveOperation::LinkIsNotDir()
 	}
 	else if (m_operationMode != recursive_list) {
 		CLocalPath localPath = dir.localDir;
-		wxString localFile = dir.subdir;
-		if (m_operationMode != recursive_addtoqueue_flatten && m_operationMode != recursive_transfer_flatten)
+		std::wstring localFile = dir.subdir;
+		if (m_operationMode != recursive_transfer_flatten) {
 			localPath.MakeParent();
-		m_pQueue->QueueFile(m_operationMode == recursive_addtoqueue || m_operationMode == recursive_addtoqueue_flatten, true, dir.subdir, (dir.subdir == localFile) ? wxString() : localFile, localPath, dir.parent, *pServer, -1);
-		m_pQueue->QueueFile_Finish(m_operationMode != recursive_addtoqueue);
+		}
+		m_pQueue->QueueFile(!m_immediate, true, dir.subdir, (dir.subdir == localFile) ? std::wstring() : localFile, localPath, dir.parent, site, -1);
+		m_pQueue->QueueFile_Finish(m_immediate);
 	}
 
 	NextOperation();
