@@ -13,10 +13,10 @@ public:
 	recursion_root() = default;
 	recursion_root(CServerPath const& start_dir, bool allow_parent);
 
-	void add_dir_to_visit(CServerPath const& path, wxString const& subdir, CLocalPath const& localDir = CLocalPath(), bool is_link = false);
+	void add_dir_to_visit(CServerPath const& path, std::wstring const& subdir, CLocalPath const& localDir = CLocalPath(), bool is_link = false);
 
 	// Queue a directory but restrict processing to the named subdirectory
-	void add_dir_to_visit_restricted(CServerPath const& path, wxString const& restrict, bool recurse);
+	void add_dir_to_visit_restricted(CServerPath const& path, std::wstring const& restrict, bool recurse);
 
 	bool empty() const { return m_dirsToVisit.empty(); }
 
@@ -27,9 +27,9 @@ private:
 	{
 	public:
 		CServerPath parent;
-		wxString subdir;
+		std::wstring subdir;
 		CLocalPath localDir;
-		fz::sparse_optional<wxString> restrict;
+		fz::sparse_optional<std::wstring> restrict;
 
 		// Symlink target might be outside actual start dir. Yet
 		// sometimes user wants to download symlink target contents
@@ -58,7 +58,7 @@ public:
 	virtual ~CRemoteRecursiveOperation();
 
 	void AddRecursionRoot(recursion_root && root);
-	void StartRecursiveOperation(OperationMode mode, std::vector<CFilter> const& filters, CServerPath const& finalDir);
+	void StartRecursiveOperation(OperationMode mode, ActiveFilters const& filters, CServerPath const& finalDir, bool immediate = true);
 
 	// Needed for recursive_chmod
 	void SetChmodDialog(CChmodDialog* pChmodDialog);
@@ -74,7 +74,7 @@ protected:
 
 	bool NextOperation();
 
-	virtual void OnStateChange(t_statechange_notifications notification, const wxString&, const void* data2);
+	virtual void OnStateChange(t_statechange_notifications notification, std::wstring const&, const void* data2) override;
 
 	bool BelowRecursionRoot(const CServerPath& path, recursion_root::new_dir &dir);
 
