@@ -19,7 +19,7 @@ bool COptionsPagePasswords::LoadPage()
 		xrc_call(*this, "ID_MASTERPASSWORD", &wxControl::Enable, checked);
 		xrc_call(*this, "ID_MASTERPASSWORD_REPEAT", &wxControl::Enable, checked);
 
-	};	
+	};
 	XRCCTRL(*this, "ID_PASSWORDS_SAVE", wxEvtHandler)->Bind(wxEVT_RADIOBUTTON, onChange);
 	XRCCTRL(*this, "ID_PASSWORDS_NOSAVE", wxEvtHandler)->Bind(wxEVT_RADIOBUTTON, onChange);
 	XRCCTRL(*this, "ID_PASSWORDS_USEMASTERPASSWORD", wxEvtHandler)->Bind(wxEVT_RADIOBUTTON, onChange);
@@ -36,7 +36,7 @@ bool COptionsPagePasswords::LoadPage()
 			xrc_call(*this, "ID_PASSWORDS_NOSAVE", &wxRadioButton::SetValue, true);
 		}
 		else {
-			auto key = public_key::from_base64(fz::to_utf8(m_pOptions->GetOption(OPTION_MASTERPASSWORDENCRYPTOR)));
+			auto key = fz::public_key::from_base64(fz::to_utf8(m_pOptions->GetOption(OPTION_MASTERPASSWORDENCRYPTOR)));
 			if (key) {
 				xrc_call(*this, "ID_PASSWORDS_USEMASTERPASSWORD", &wxRadioButton::SetValue, true);
 
@@ -60,7 +60,7 @@ bool COptionsPagePasswords::SavePage()
 		return true;
 	}
 
-	auto oldPub = public_key::from_base64(fz::to_utf8(m_pOptions->GetOption(OPTION_MASTERPASSWORDENCRYPTOR)));
+	auto oldPub = fz::public_key::from_base64(fz::to_utf8(m_pOptions->GetOption(OPTION_MASTERPASSWORDENCRYPTOR)));
 	wxString const newPw = xrc_call(*this, "ID_MASTERPASSWORD", &wxTextCtrl::GetValue);
 
 	bool useMaster = xrc_call(*this, "ID_PASSWORDS_USEMASTERPASSWORD", &wxRadioButton::GetValue);
@@ -77,10 +77,10 @@ bool COptionsPagePasswords::SavePage()
 	}
 
 	if (useMaster) {
-		auto priv = private_key::from_password(fz::to_utf8(newPw), fz::random_bytes(private_key::salt_size));
+		auto priv = fz::private_key::from_password(fz::to_utf8(newPw), fz::random_bytes(fz::private_key::salt_size));
 		auto pub = priv.pubkey();
 		if (!pub) {
-			wxMessageBox(_("Could not generate key"), _("Error"));
+			wxMessageBoxEx(_("Could not generate key"), _("Error"));
 		}
 		else {
 			m_pOptions->SetOption(OPTION_DEFAULT_KIOSKMODE, 0);
@@ -96,9 +96,9 @@ bool COptionsPagePasswords::SavePage()
 	// Now actually change stored passwords
 	{
 		auto recentServers = CRecentServerList::GetMostRecentServers();
-		for (auto & server : recentServers) {
-			loginManager.AskDecryptor(server.credentials.encrypted_, true, false);
-			server.credentials.Unprotect(loginManager.GetDecryptor(server.credentials.encrypted_), true);
+		for (auto & site : recentServers) {
+			loginManager.AskDecryptor(site.credentials.encrypted_, true, false);
+			site.credentials.Unprotect(loginManager.GetDecryptor(site.credentials.encrypted_), true);
 		}
 		CRecentServerList::SetMostRecentServers(recentServers);
 	}
@@ -106,8 +106,8 @@ bool COptionsPagePasswords::SavePage()
 	for (auto state : *CContextManager::Get()->GetAllStates()) {
 		auto site = state->GetLastSite();
 		auto path = state->GetLastServerPath();
-		loginManager.AskDecryptor(site.server_.credentials.encrypted_, true, false);
-		site.server_.credentials.Unprotect(loginManager.GetDecryptor(site.server_.credentials.encrypted_), true);
+		loginManager.AskDecryptor(site.credentials.encrypted_, true, false);
+		site.credentials.Unprotect(loginManager.GetDecryptor(site.credentials.encrypted_), true);
 		state->SetLastSite(site, path);
 	}
 
@@ -115,7 +115,7 @@ bool COptionsPagePasswords::SavePage()
 
 	CSiteManager::Rewrite(loginManager, true);
 
-	CContextManager::Get()->NotifyGlobalHandlers(STATECHANGE_REWRITE_CREDENTIALS, wxString(), &loginManager);
+	CContextManager::Get()->NotifyGlobalHandlers(STATECHANGE_REWRITE_CREDENTIALS, std::wstring(), &loginManager);
 
 	return true;
 }
@@ -130,7 +130,7 @@ bool COptionsPagePasswords::Validate()
 			return DisplayError(_T("ID_MASTERPASSWORD"), _("The entered passwords are not the same."));
 		}
 
-		auto key = public_key::from_base64(fz::to_utf8(m_pOptions->GetOption(OPTION_MASTERPASSWORDENCRYPTOR)));
+		auto key = fz::public_key::from_base64(fz::to_utf8(m_pOptions->GetOption(OPTION_MASTERPASSWORDENCRYPTOR)));
 		if (!key && pw.empty()) {
 			return DisplayError(_T("ID_MASTERPASSWORD"), _("You need to enter a master password."));
 		}
@@ -173,4 +173,4 @@ bool COptionsPagePasswords::CreateControls(wxWindow* parent)
 	SetSizer(outer);
 
 	return true;
-}
\ No newline at end of file
+}
