@@ -1,85 +1,32 @@
 #include <filezilla.h>
 #include "sitemanager.h"
 
+#include "dialogex.h"
 #include "filezillaapp.h"
 #include "ipcmutex.h"
+#include "loginmanager.h"
 #include "Options.h"
 #include "xmlfunctions.h"
 
 namespace {
 struct background_color {
 	wxColour const color;
-	wchar_t const*const name;
+	char const*const name;
 };
 
 background_color const background_colors[] = {
-	{ wxColour(), TRANSLATE_T("None") },
-	{ wxColour(255, 0, 0, 32), TRANSLATE_T("Red") },
-	{ wxColour(0, 255, 0, 32), TRANSLATE_T("Green") },
-	{ wxColour(0, 0, 255, 32), TRANSLATE_T("Blue") },
-	{ wxColour(255, 255, 0, 32), TRANSLATE_T("Yellow") },
-	{ wxColour(0, 255, 255, 32), TRANSLATE_T("Cyan") },
-	{ wxColour(255, 0, 255, 32), TRANSLATE_T("Magenta") },
-	{ wxColour(255, 128, 0, 32), TRANSLATE_T("Orange") },
+	{ wxColour(), fztranslate_mark("None") },
+	{ wxColour(255, 0, 0, 32), fztranslate_mark("Red") },
+	{ wxColour(0, 255, 0, 32), fztranslate_mark("Green") },
+	{ wxColour(0, 0, 255, 32), fztranslate_mark("Blue") },
+	{ wxColour(255, 255, 0, 32), fztranslate_mark("Yellow") },
+	{ wxColour(0, 255, 255, 32), fztranslate_mark("Cyan") },
+	{ wxColour(255, 0, 255, 32), fztranslate_mark("Magenta") },
+	{ wxColour(255, 128, 0, 32), fztranslate_mark("Orange") },
 	{ wxColour(), 0 }
 };
 }
 
-bool Bookmark::operator==(Bookmark const& b) const
-{
-	if (m_localDir != b.m_localDir) {
-		return false;
-	}
-	
-	if (m_remoteDir != b.m_remoteDir) {
-		return false;
-	}
-
-	if (m_sync != b.m_sync) {
-		return false;
-	}
-
-	if (m_comparison != b.m_comparison) {
-		return false;
-	}
-
-	if (m_name != b.m_name) {
-		return false;
-	}
-
-	return true;
-}
-
-bool Site::operator==(Site const& s) const
-{
-	if (m_server != s.m_server) {
-		return false;
-	}
-
-	if (m_comments != s.m_comments) {
-		return false;
-	}
-
-	if (m_default_bookmark != s.m_default_bookmark) {
-		return false;
-	}
-
-	if (m_bookmarks != s.m_bookmarks) {
-		return false;
-	}
-
-	if (m_path != s.m_path) {
-		return false;
-	}
-
-	if (m_colour != s.m_colour) {
-		return false;
-	}
-
-	return true;
-}
-
-
 std::map<int, std::unique_ptr<Site>> CSiteManager::m_idMap;
 
 bool CSiteManager::Load(CSiteManagerXmlHandler& handler)
@@ -151,23 +98,22 @@ bool CSiteManager::ReadBookmarkElement(Bookmark & bookmark, pugi::xml_node eleme
 
 std::unique_ptr<Site> CSiteManager::ReadServerElement(pugi::xml_node element)
 {
-	CServer server;
-	if (!::GetServer(element, server))
-		return 0;
-	if (server.GetName().empty())
-		return 0;
-
 	auto data = std::make_unique<Site>();
-	data->m_server = server;
+	if (!::GetServer(element, *data)) {
+		return 0;
+	}
+	if (data->server.GetName().empty()) {
+		return 0;
+	}
 
-	data->m_comments = GetTextElement(element, "Comments");
+	data->comments_ = GetTextElement(element, "Comments");
 	data->m_colour = GetColourFromIndex(GetTextElementInt(element, "Colour"));
-	
+
 	ReadBookmarkElement(data->m_default_bookmark, element);
 
 	// Bookmarks
 	for (auto bookmark = element.child("Bookmark"); bookmark; bookmark = bookmark.next_sibling("Bookmark")) {
-		wxString name = GetTextElement_Trimmed(bookmark, "Name");
+		std::wstring name = GetTextElement_Trimmed(bookmark, "Name");
 		if (name.empty()) {
 			continue;
 		}
@@ -234,12 +180,12 @@ public:
 
 	virtual bool AddSite(std::unique_ptr<Site> data)
 	{
-		wxString newName(data->m_server.GetName());
+		std::wstring newName(data->server.GetName());
 		int i = GetInsertIndex(m_pMenu, newName);
-		newName.Replace(_T("&"), _T("&&"));
+		newName = LabelEscape(newName);
 		wxMenuItem* pItem = m_pMenu->Insert(i, wxID_ANY, newName);
 
-		data->m_path = path + _T("/") + CSiteManager::EscapeSegment(data->m_server.GetName());
+		data->SetSitePath(path + _T("/") + CSiteManager::EscapeSegment(data->server.GetName()));
 
 		(*m_idMap)[pItem->GetId()] = std::move(data);
 
@@ -256,9 +202,9 @@ public:
 		wxMenu* pChild = m_pMenu;
 		m_pMenu = m_parents.back();
 		if (pChild->GetMenuItemCount()) {
-			wxString name = m_childNames.back();
+			std::wstring name = m_childNames.back();
 			int i = GetInsertIndex(m_pMenu, name);
-			name.Replace(_T("&"), _T("&&"));
+			name = LabelEscape(name);
 
 			wxMenuItem* pItem = new wxMenuItem(m_pMenu, wxID_ANY, name, _T(""), wxITEM_NORMAL, pChild);
 			m_pMenu->Insert(i, pItem);
@@ -330,8 +276,9 @@ std::unique_ptr<wxMenu> CSiteManager::GetSitesMenu()
 	}
 
 	if (pMenu) {
-		if (!predefinedSites)
+		if (!predefinedSites) {
 			return pMenu;
+		}
 
 		auto pRootMenu = std::make_unique<wxMenu>();
 		pRootMenu->AppendSubMenu(predefinedSites.release(), _("Predefined Sites"));
@@ -340,8 +287,9 @@ std::unique_ptr<wxMenu> CSiteManager::GetSitesMenu()
 		return pRootMenu;
 	}
 
-	if (predefinedSites)
+	if (predefinedSites) {
 		return predefinedSites;
+	}
 
 	pMenu = std::make_unique<wxMenu>();
 	wxMenuItem* pItem = pMenu->Append(wxID_ANY, _("No sites available"));
@@ -357,19 +305,22 @@ void CSiteManager::ClearIdMap()
 bool CSiteManager::LoadPredefined(CSiteManagerXmlHandler& handler)
 {
 	CLocalPath const defaultsDir = wxGetApp().GetDefaultsDir();
-	if (defaultsDir.empty())
+	if (defaultsDir.empty()) {
 		return false;
+	}
 
-	wxString const name(defaultsDir.GetPath() + _T("fzdefaults.xml"));
+	std::wstring const name(defaultsDir.GetPath() + _T("fzdefaults.xml"));
 	CXmlFile file(name);
 
 	auto document = file.Load();
-	if (!document)
+	if (!document) {
 		return false;
+	}
 
 	auto element = document.child("Servers");
-	if (!element)
+	if (!element) {
 		return false;
+	}
 
 	if (!Load(element, handler)) {
 		return false;
@@ -558,13 +509,13 @@ std::pair<std::unique_ptr<Site>, Bookmark> CSiteManager::DoGetSiteByPath(std::ws
 			ret.second = ret.first->m_default_bookmark;
 		}
 
-		ret.first->m_path = BuildPath(c, segments);
+		ret.first->SetSitePath(BuildPath(c, segments));
 	}
 
 	return ret;
 }
 
-wxString CSiteManager::AddServer(CServer server)
+std::wstring CSiteManager::AddServer(Site site)
 {
 	// We have to synchronize access to sitemanager.xml so that multiple processed don't write
 	// to the same file or one is reading while the other one writes.
@@ -576,53 +527,58 @@ wxString CSiteManager::AddServer(CServer server)
 		wxString msg = file.GetError() + _T("\n") + _("The server could not be added.");
 		wxMessageBoxEx(msg, _("Error loading xml file"), wxICON_ERROR);
 
-		return wxString();
+		return std::wstring();
 	}
 
 	auto element = document.child("Servers");
-	if (!element)
-		return wxString();
+	if (!element) {
+		element = document.append_child("Servers");
+	}
 
-	std::list<wxString> names;
+	std::vector<std::wstring> names;
 	for (auto child = element.child("Server"); child; child = child.next_sibling("Server")) {
-		wxString name = GetTextElement(child, "Name");
-		if (name.empty())
+		std::wstring name = GetTextElement(child, "Name");
+		if (name.empty()) {
 			continue;
+		}
 
 		names.push_back(name);
 	}
 
-	std::wstring name = _("New site").ToStdWstring();
+	std::wstring name = fztranslate("New site");
 	int i = 1;
 
 	for (;;) {
-		std::list<wxString>::const_iterator iter;
-		for (iter = names.begin(); iter != names.end(); ++iter) {
-			if (*iter == name)
+		std::vector<std::wstring>::const_iterator iter;
+		for (iter = names.cbegin(); iter != names.cend(); ++iter) {
+			if (*iter == name) {
 				break;
+			}
 		}
-		if (iter == names.end())
+		if (iter == names.cend()) {
 			break;
+		}
 
-		name = _("New site") + wxString::Format(_T(" %d"), ++i);
+		name = _("New site").ToStdWstring() + fz::sprintf(L" %d", ++i);
 	}
 
-	server.SetName(name);
+	site.server.SetName(name);
 
 	auto xServer = element.append_child("Server");
-	SetServer(xServer, server);
+	SetServer(xServer, site);
 	AddTextElement(xServer, name);
 
 	if (!file.Save(false)) {
-		if (COptions::Get()->GetOptionVal(OPTION_DEFAULT_KIOSKMODE) == 2)
-			return wxString();
+		if (COptions::Get()->GetOptionVal(OPTION_DEFAULT_KIOSKMODE) == 2) {
+			return std::wstring();
+		}
 
-		wxString msg = wxString::Format(_("Could not write \"%s\", any changes to the Site Manager could not be saved: %s"), file.GetFileName(), file.GetError());
+		std::wstring msg = fz::sprintf(fztranslate("Could not write \"%s\", any changes to the Site Manager could not be saved: %s"), file.GetFileName(), file.GetError());
 		wxMessageBoxEx(msg, _("Error writing xml file"), wxICON_ERROR);
-		return wxString();
+		return std::wstring();
 	}
 
-	return _T("0/") + EscapeSegment(name);
+	return L"0/" + EscapeSegment(name);
 }
 
 pugi::xml_node CSiteManager::GetElementByPath(pugi::xml_node node, std::vector<std::wstring> const& segments)
@@ -630,20 +586,25 @@ pugi::xml_node CSiteManager::GetElementByPath(pugi::xml_node node, std::vector<s
 	for (auto const& segment : segments) {
 		pugi::xml_node child;
 		for (child = node.first_child(); child; child = child.next_sibling()) {
-			if (strcmp(child.name(), "Server") && strcmp(child.name(), "Folder") && strcmp(child.name(), "Bookmark"))
+			if (strcmp(child.name(), "Server") && strcmp(child.name(), "Folder") && strcmp(child.name(), "Bookmark")) {
 				continue;
+			}
 
 			wxString name = GetTextElement_Trimmed(child, "Name");
-			if (name.empty())
+			if (name.empty()) {
 				name = GetTextElement_Trimmed(child);
-			if (name.empty())
+			}
+			if (name.empty()) {
 				continue;
+			}
 
-			if (name == segment)
+			if (name == segment) {
 				break;
+			}
 		}
-		if (!child)
+		if (!child) {
 			return pugi::xml_node();
+		}
 
 		node = child;
 		continue;
@@ -654,12 +615,14 @@ pugi::xml_node CSiteManager::GetElementByPath(pugi::xml_node node, std::vector<s
 
 bool CSiteManager::AddBookmark(std::wstring sitePath, const wxString& name, const wxString &local_dir, const CServerPath &remote_dir, bool sync, bool comparison)
 {
-	if (local_dir.empty() && remote_dir.empty())
+	if (local_dir.empty() && remote_dir.empty()) {
 		return false;
+	}
 
-	wxChar c = sitePath.empty() ? 0 : sitePath[0];
-	if (c != '0')
+	auto const c = sitePath[0];
+	if (c != '0') {
 		return false;
+	}
 
 	sitePath = sitePath.substr(1);
 
@@ -677,8 +640,9 @@ bool CSiteManager::AddBookmark(std::wstring sitePath, const wxString& name, cons
 	}
 
 	auto element = document.child("Servers");
-	if (!element)
+	if (!element) {
 		return false;
+	}
 
 	std::vector<std::wstring> segments;
 	if (!UnescapeSitePath(sitePath, segments)) {
@@ -695,35 +659,44 @@ bool CSiteManager::AddBookmark(std::wstring sitePath, const wxString& name, cons
 	// Bookmarks
 	pugi::xml_node insertBefore, bookmark;
 	for (bookmark = child.child("Bookmark"); bookmark; bookmark = bookmark.next_sibling("Bookmark")) {
-		wxString old_name = GetTextElement_Trimmed(bookmark, "Name");
-		if (old_name.empty())
+		std::wstring old_name = GetTextElement_Trimmed(bookmark, "Name");
+		if (old_name.empty()) {
 			continue;
+		}
 
 		if (name == old_name) {
 			wxMessageBoxEx(_("Name of bookmark already exists."), _("New bookmark"), wxICON_EXCLAMATION);
 			return false;
 		}
-		if (name < old_name && !insertBefore)
+		if (name < old_name && !insertBefore) {
 			insertBefore = bookmark;
+		}
 	}
 
-	if (insertBefore)
+	if (insertBefore) {
 		bookmark = child.insert_child_before("Bookmark", insertBefore);
-	else
+	}
+	else {
 		bookmark = child.append_child("Bookmark");
-	AddTextElement(bookmark, "Name", name);
-	if (!local_dir.empty())
-		AddTextElement(bookmark, "LocalDir", local_dir);
-	if (!remote_dir.empty())
+	}
+	AddTextElement(bookmark, "Name", name.ToStdWstring());
+	if (!local_dir.empty()) {
+		AddTextElement(bookmark, "LocalDir", local_dir.ToStdWstring());
+	}
+	if (!remote_dir.empty()) {
 		AddTextElement(bookmark, "RemoteDir", remote_dir.GetSafePath());
-	if (sync)
-		AddTextElementRaw(bookmark, "SyncBrowsing", "1");
-	if (comparison)
-		AddTextElementRaw(bookmark, "DirectoryComparison", "1");
+	}
+	if (sync) {
+		AddTextElementUtf8(bookmark, "SyncBrowsing", "1");
+	}
+	if (comparison) {
+		AddTextElementUtf8(bookmark, "DirectoryComparison", "1");
+	}
 
 	if (!file.Save(false)) {
-		if (COptions::Get()->GetOptionVal(OPTION_DEFAULT_KIOSKMODE) == 2)
+		if (COptions::Get()->GetOptionVal(OPTION_DEFAULT_KIOSKMODE) == 2) {
 			return true;
+		}
 
 		wxString msg = wxString::Format(_("Could not write \"%s\", the selected sites could not be exported: %s"), file.GetFileName(), file.GetError());
 		wxMessageBoxEx(msg, _("Error writing xml file"), wxICON_ERROR);
@@ -735,8 +708,9 @@ bool CSiteManager::AddBookmark(std::wstring sitePath, const wxString& name, cons
 bool CSiteManager::ClearBookmarks(std::wstring sitePath)
 {
 	wxChar const c = sitePath.empty() ? 0 : sitePath[0];
-	if (c != '0')
+	if (c != '0') {
 		return false;
+	}
 
 	sitePath = sitePath.substr(1);
 
@@ -754,8 +728,9 @@ bool CSiteManager::ClearBookmarks(std::wstring sitePath)
 	}
 
 	auto element = document.child("Servers");
-	if (!element)
+	if (!element) {
 		return false;
+	}
 
 	std::vector<std::wstring> segments;
 	if (!UnescapeSitePath(sitePath, segments)) {
@@ -776,8 +751,9 @@ bool CSiteManager::ClearBookmarks(std::wstring sitePath)
 	}
 
 	if (!file.Save(false)) {
-		if (COptions::Get()->GetOptionVal(OPTION_DEFAULT_KIOSKMODE) == 2)
+		if (COptions::Get()->GetOptionVal(OPTION_DEFAULT_KIOSKMODE) == 2) {
 			return true;
+		}
 
 		wxString msg = wxString::Format(_("Could not write \"%s\", the selected sites could not be exported: %s"), file.GetFileName(), file.GetError());
 		wxMessageBoxEx(msg, _("Error writing xml file"), wxICON_ERROR);
@@ -821,3 +797,78 @@ wxString CSiteManager::GetColourName(int i)
 	return wxGetTranslation(background_colors[i].name);
 }
 
+void CSiteManager::Rewrite(CLoginManager & loginManager, pugi::xml_node element, bool on_failure_set_to_ask)
+{
+	for (auto child = element.first_child(); child; child = child.next_sibling()) {
+		if (!strcmp(child.name(), "Folder")) {
+			Rewrite(loginManager, child, on_failure_set_to_ask);
+		}
+		else if (!strcmp(child.name(), "Server")) {
+			auto site = ReadServerElement(child);
+			if (site) {
+				loginManager.AskDecryptor(site->credentials.encrypted_, true, false);
+				site->credentials.Unprotect(loginManager.GetDecryptor(site->credentials.encrypted_), on_failure_set_to_ask);
+				Save(child, *site);
+			}
+		}
+	}
+}
+
+void CSiteManager::Rewrite(CLoginManager & loginManager, bool on_failure_set_to_ask)
+{
+	if (COptions::Get()->GetOptionVal(OPTION_DEFAULT_KIOSKMODE) == 2) {
+		return;
+	}
+	CInterProcessMutex mutex(MUTEX_SITEMANAGER);
+
+	CXmlFile file(wxGetApp().GetSettingsFile(_T("sitemanager")));
+	auto document = file.Load();
+	if (!document) {
+		wxMessageBoxEx(file.GetError(), _("Error loading xml file"), wxICON_ERROR);
+		return;
+	}
+
+	auto element = document.child("Servers");
+	if (!element) {
+		return;
+	}
+
+	Rewrite(loginManager, element, on_failure_set_to_ask);
+
+	file.Save(true);
+}
+
+void CSiteManager::Save(pugi::xml_node element, Site const& site)
+{
+	SetServer(element, site);
+
+	// Save comments
+	AddTextElement(element, "Comments", site.comments_);
+
+	// Save colour
+	AddTextElement(element, "Colour", CSiteManager::GetColourIndex(site.m_colour));
+
+	// Save local dir
+	AddTextElement(element, "LocalDir", site.m_default_bookmark.m_localDir);
+
+	// Save remote dir
+	AddTextElement(element, "RemoteDir", site.m_default_bookmark.m_remoteDir.GetSafePath());
+
+	AddTextElementUtf8(element, "SyncBrowsing", site.m_default_bookmark.m_sync ? "1" : "0");
+	AddTextElementUtf8(element, "DirectoryComparison", site.m_default_bookmark.m_comparison ? "1" : "0");
+
+	for (auto const& bookmark : site.m_bookmarks) {
+		auto node = element.append_child("Bookmark");
+
+		AddTextElement(node, "Name", bookmark.m_name);
+
+		// Save local dir
+		AddTextElement(node, "LocalDir", bookmark.m_localDir);
+
+		// Save remote dir
+		AddTextElement(node, "RemoteDir", bookmark.m_remoteDir.GetSafePath());
+
+		AddTextElementUtf8(node, "SyncBrowsing", bookmark.m_sync ? "1" : "0");
+		AddTextElementUtf8(node, "DirectoryComparison", bookmark.m_comparison ? "1" : "0");
+	}
+}
