@@ -16,7 +16,7 @@
 
 #include <algorithm>
 
-bool CStateFilterManager::FilenameFiltered(std::wstring const& name, const wxString& path, bool dir, int64_t size, bool local, int attributes, fz::datetime const& date) const
+bool CStateFilterManager::FilenameFiltered(std::wstring const& name, std::wstring const& path, bool dir, int64_t size, bool local, int attributes, fz::datetime const& date) const
 {
 	if (local) {
 		if (m_localFilter && FilenameFilteredByFilter(m_localFilter, name, path, dir, size, attributes, date)) {
@@ -161,7 +161,7 @@ size_t CContextManager::HandlerCount(t_statechange_notifications notification) c
 	return m_handlers[notification].size();
 }
 
-void CContextManager::NotifyHandlers(CState* pState, t_statechange_notifications notification, wxString const& data, void const* data2)
+void CContextManager::NotifyHandlers(CState* pState, t_statechange_notifications notification, std::wstring const& data, void const* data2)
 {
 	wxASSERT(notification != STATECHANGE_NONE && notification != STATECHANGE_MAX);
 
@@ -184,14 +184,14 @@ CState* CContextManager::GetCurrentContext()
 	return m_contexts[m_current_context];
 }
 
-void CContextManager::NotifyAllHandlers(t_statechange_notifications notification, wxString const& data, void const* data2)
+void CContextManager::NotifyAllHandlers(t_statechange_notifications notification, std::wstring const& data, void const* data2)
 {
 	for (auto const& context : m_contexts) {
 		context->NotifyHandlers(notification, data, data2);
 	}
 }
 
-void CContextManager::NotifyGlobalHandlers(t_statechange_notifications notification, wxString const& data, void const* data2)
+void CContextManager::NotifyGlobalHandlers(t_statechange_notifications notification, std::wstring const& data, void const* data2)
 {
 	auto const& handlers = m_handlers[notification];
 	for (auto const& handler : handlers) {
@@ -205,7 +205,7 @@ void CContextManager::ProcessDirectoryListing(CServer const& server, std::shared
 		if (state == exempt) {
 			continue;
 		}
-		if (state->GetServer() && state->GetServer().server == server) {
+		if (state->GetSite() && state->GetSite().server == server) {
 			state->SetRemoteDir(listing, false);
 		}
 	}
@@ -275,7 +275,7 @@ bool CState::SetLocalDir(CLocalPath const& dir, std::wstring *error, bool rememb
 	}
 
 	if (!m_sync_browse.local_root.empty()) {
-		wxASSERT(m_site.server_);
+		wxASSERT(m_site);
 
 		if (dir != m_sync_browse.local_root && !dir.IsSubdirOf(m_sync_browse.local_root)) {
 			wxString msg = wxString::Format(_("The local directory '%s' is not below the synchronization root (%s).\nDisable synchronized browsing and continue changing the local directory?"),
@@ -348,7 +348,7 @@ bool CState::SetRemoteDir(std::shared_ptr<CDirectoryListing> const& pDirectoryLi
 
 		if (m_pDirectoryListing) {
 			m_pDirectoryListing = 0;
-			NotifyHandlers(STATECHANGE_REMOTE_DIR, wxString(), &primary);
+			NotifyHandlers(STATECHANGE_REMOTE_DIR, std::wstring(), &primary);
 		}
 		m_previouslyVisitedRemoteSubdir.clear();
 		return true;
@@ -384,7 +384,7 @@ bool CState::SetRemoteDir(std::shared_ptr<CDirectoryListing> const& pDirectoryLi
 
 	m_pDirectoryListing = pDirectoryListing;
 
-	NotifyHandlers(STATECHANGE_REMOTE_DIR, wxString(), &primary);
+	NotifyHandlers(STATECHANGE_REMOTE_DIR, std::wstring(), &primary);
 
 	bool compare = m_changeDirFlags.compare;
 	if (primary) {
@@ -481,9 +481,9 @@ void CState::LocalDirCreated(const CLocalPath& path)
 		return;
 	}
 
-	wxString next_segment = path.GetPath().substr(m_localDir.GetPath().size());
-	int pos = next_segment.Find(CLocalPath::path_separator);
-	if (pos <= 0) {
+	std::wstring next_segment = path.GetPath().substr(m_localDir.GetPath().size());
+	size_t pos = next_segment.find(CLocalPath::path_separator);
+	if (pos == std::wstring::npos || !pos) {
 		// Shouldn't ever come true
 		return;
 	}
@@ -491,17 +491,14 @@ void CState::LocalDirCreated(const CLocalPath& path)
 	// Current local path is /foo/
 	// Called with /foo/bar/baz/
 	// -> Refresh /foo/bar/
-	next_segment = next_segment.Left(pos);
+	next_segment = next_segment.substr(0, pos);
 	NotifyHandlers(STATECHANGE_LOCAL_REFRESH_FILE, next_segment);
 }
 
 void CState::SetSite(Site const& site, CServerPath const& path)
 {
-	if (m_site.server_) {
-		if (site.server_ && site.server_ == m_site.server_ &&
-			site.server_.server.GetName() == m_site.server_.server.GetName() &&
-			site.server_.server.MaximumMultipleConnections() == m_site.server_.server.MaximumMultipleConnections())
-		{
+	if (m_site) {
+		if (site == m_site) {
 			// Nothing changes
 			return;
 		}
@@ -510,11 +507,11 @@ void CState::SetSite(Site const& site, CServerPath const& path)
 		m_pCertificate.reset();
 		m_pSftpEncryptionInfo.reset();
 	}
-	if (site.server_) {
+	if (site) {
 		if (!path.empty()) {
 			m_last_path = path;
 		}
-		else if (m_last_site.server_ != site.server_) {
+		else if (m_last_site.server != site.server) {
 			m_last_path.clear();
 		}
 		m_last_site = site;
@@ -534,11 +531,6 @@ Site const& CState::GetSite() const
 	return m_site;
 }
 
-ServerWithCredentials const& CState::GetServer() const
-{
-	return m_site.server_;
-}
-
 wxString CState::GetTitle() const
 {
 	return m_title;
@@ -546,7 +538,7 @@ wxString CState::GetTitle() const
 
 bool CState::Connect(Site const& site, CServerPath const& path, bool compare)
 {
-	if (!site.server_) {
+	if (!site) {
 		return false;
 	}
 	if (!m_pEngine) {
@@ -557,13 +549,13 @@ bool CState::Connect(Site const& site, CServerPath const& path, bool compare)
 	}
 	m_pRemoteRecursiveOperation->StopRecursiveOperation();
 	SetSyncBrowse(false);
-
-	m_pCommandQueue->ProcessCommand(new CConnectCommand(site.server_.server, site.server_.credentials));
-	m_pCommandQueue->ProcessCommand(new CListCommand(path, std::wstring(), LIST_FLAG_FALLBACK_CURRENT));
+	m_changeDirFlags.compare = compare;
 
 	SetSite(site, path);
 
-	m_changeDirFlags.compare = compare;
+	// Use m_site from here on
+	m_pCommandQueue->ProcessCommand(new CConnectCommand(m_site.server, m_site.Handle(), m_site.credentials));
+	m_pCommandQueue->ProcessCommand(new CListCommand(path, std::wstring(), LIST_FLAG_FALLBACK_CURRENT));
 
 	return true;
 }
@@ -679,7 +671,7 @@ void CState::UnregisterHandler(CStateEventHandler* pHandler, t_statechange_notif
 	}
 }
 
-void CState::NotifyHandlers(t_statechange_notifications notification, const wxString& data, const void* data2)
+void CState::NotifyHandlers(t_statechange_notifications notification, std::wstring const& data, const void* data2)
 {
 	wxASSERT(notification != STATECHANGE_NONE && notification != STATECHANGE_MAX);
 
@@ -720,35 +712,35 @@ CGlobalStateEventHandler::~CGlobalStateEventHandler()
 	CContextManager::Get()->UnregisterHandler(this, STATECHANGE_NONE);
 }
 
-void CState::UploadDroppedFiles(CLocalDataObject const* pLocalDataObject, const wxString& subdir, bool queueOnly)
+void CState::UploadDroppedFiles(CLocalDataObject const* pLocalDataObject, std::wstring const& subdir, bool queueOnly)
 {
-	if (!m_site.server_ || !m_pDirectoryListing) {
+	if (!m_site || !m_pDirectoryListing) {
 		return;
 	}
 
 	CServerPath path = m_pDirectoryListing->path;
-	if (subdir == _T("..") && path.HasParent()) {
+	if (subdir == L".." && path.HasParent()) {
 		path = path.GetParent();
 	}
 	else if (!subdir.empty()) {
-		path.AddSegment(subdir.ToStdWstring());
+		path.AddSegment(subdir);
 	}
 
 	UploadDroppedFiles(pLocalDataObject, path, queueOnly);
 }
 
-void CState::UploadDroppedFiles(const wxFileDataObject* pFileDataObject, const wxString& subdir, bool queueOnly)
+void CState::UploadDroppedFiles(const wxFileDataObject* pFileDataObject, std::wstring const& subdir, bool queueOnly)
 {
-	if (!m_site.server_ || !m_pDirectoryListing) {
+	if (!m_site || !m_pDirectoryListing) {
 		return;
 	}
 
 	CServerPath path = m_pDirectoryListing->path;
-	if (subdir == _T("..") && path.HasParent()) {
+	if (subdir == L".." && path.HasParent()) {
 		path = path.GetParent();
 	}
 	else if (!subdir.empty()) {
-		path.AddSegment(subdir.ToStdWstring());
+		path.AddSegment(subdir);
 	}
 
 	UploadDroppedFiles(pFileDataObject, path, queueOnly);
@@ -762,7 +754,7 @@ void DoUploadDroppedFiles(CState& state, CMainFrame & mainFrame, T const& files,
 		return;
 	}
 
-	if (!state.GetServer()) {
+	if (!state.GetSite()) {
 		return;
 	}
 
@@ -783,7 +775,7 @@ void DoUploadDroppedFiles(CState& state, CMainFrame & mainFrame, T const& files,
 		if (type == fz::local_filesys::file) {
 			std::wstring localFile;
 			CLocalPath const localPath(fz::to_wstring(file), &localFile);
-			mainFrame.GetQueue()->QueueFile(queueOnly, false, localFile, wxEmptyString, localPath, path, state.GetServer(), size);
+			mainFrame.GetQueue()->QueueFile(queueOnly, false, localFile, wxEmptyString, localPath, path, state.GetSite(), size);
 			mainFrame.GetQueue()->QueueFile_Finish(!queueOnly);
 		}
 		else if (type == fz::local_filesys::dir) {
@@ -858,6 +850,8 @@ void DoHandleDroppedFiles(CState & state, CMainFrame & mainFrame, T const& files
 	delete[] to;
 	delete[] from;
 #else
+	(void)mainFrame;
+
 	wxString error;
 	for (auto const& file : files) {
 		int64_t size;
@@ -1042,7 +1036,7 @@ bool CState::IsRemoteConnected() const
 		return false;
 	}
 
-	return static_cast<bool>(m_site.server_);
+	return static_cast<bool>(m_site);
 }
 
 bool CState::IsRemoteIdle(bool ignore_recursive) const
@@ -1103,7 +1097,7 @@ void CState::ListingFailed(int)
 	}
 }
 
-void CState::LinkIsNotDir(const CServerPath& path, const wxString& subdir)
+void CState::LinkIsNotDir(const CServerPath& path, std::wstring const& subdir)
 {
 	m_changeDirFlags.compare = false;
 	m_changeDirFlags.syncbrowse = false;
@@ -1113,7 +1107,7 @@ void CState::LinkIsNotDir(const CServerPath& path, const wxString& subdir)
 
 bool CState::ChangeRemoteDir(CServerPath const& path, std::wstring const& subdir, int flags, bool ignore_busy, bool compare)
 {
-	if (!m_site.server_ || !m_pCommandQueue) {
+	if (!m_site || !m_pCommandQueue) {
 		return false;
 	}
 
@@ -1172,7 +1166,7 @@ bool CState::ChangeRemoteDir(CServerPath const& path, std::wstring const& subdir
 					}
 
 					if (!local_path.Exists(&error)) {
-						wxMessageBox(wxString::Format(_("The local directory '%s' could not be created."), local_path.GetPath()), _("Synchronized browsing"), wxICON_EXCLAMATION);
+						wxMessageBoxEx(wxString::Format(_("The local directory '%s' could not be created."), local_path.GetPath()), _("Synchronized browsing"), wxICON_EXCLAMATION);
 						return false;
 					}
 					m_changeDirFlags.syncbrowse = true;
@@ -1328,23 +1322,24 @@ void CState::SetSecurityInfo(CSftpEncryptionNotification const& info)
 	NotifyHandlers(STATECHANGE_ENCRYPTION);
 }
 
-void CState::UpdateSite(wxString const& oldPath, Site const& newSite)
+void CState::UpdateSite(std::wstring const& oldPath, Site const& newSite)
 {
-	if (newSite.m_path.empty() || !newSite.server_) {
+	if (newSite.SitePath().empty() || !newSite) {
 		return;
 	}
 
 	bool changed = false;
-	if (m_site.server_ && m_site != newSite) {
-		if (m_site.m_path == oldPath && m_site.server_ == newSite.server_) {
-			m_site = newSite;
+	if (m_site && m_site != newSite) {
+		if (m_site.SitePath() == oldPath && m_site.server == newSite.server) {
+			// Update handles
+			m_site.Update(newSite);
 			changed = true;
 		}
 	}
-	if (m_last_site.server_ && m_last_site != newSite) {
-		if (m_last_site.m_path == oldPath && m_last_site.server_ == newSite.server_) {
-			m_last_site = newSite;
-			if (!m_site.server_) {
+	if (m_last_site && m_last_site != newSite) {
+		if (m_last_site.SitePath() == oldPath && m_last_site.server == newSite.server) {
+			m_last_site.Update(newSite);
+			if (!m_site) {
 				// Active site has precedence over historic data
 				changed = true;
 			}
@@ -1356,15 +1351,63 @@ void CState::UpdateSite(wxString const& oldPath, Site const& newSite)
 	}
 }
 
+void CState::UpdateKnownSites(std::vector<CSiteManagerDialog::_connected_site> const& active_sites)
+{
+	bool changed{};
+	if (m_site) {
+		for (auto const& active_site : active_sites) {
+			if (active_site.old_path == m_site.SitePath()) {
+				std::unique_ptr<Site> newSite = CSiteManager::GetSiteByPath(active_site.new_path, false).first;
+
+				if (active_site.old_path == m_site.SitePath() && newSite && m_site.SameResource(*newSite)) {
+					if (m_site != *newSite) {
+						changed = true;
+						m_site.Update(*newSite);
+					}
+				}
+				else {
+					changed = true;
+					m_site.SetSitePath(std::wstring());
+				}
+				m_last_site = m_site;
+				break;
+			}
+		}
+	}
+	else if (m_last_site) {
+		for (auto const& active_site : active_sites) {
+			if (active_site.old_path == m_last_site.SitePath()) {
+				std::unique_ptr<Site> newSite = CSiteManager::GetSiteByPath(active_site.new_path, false).first;
+				if (active_site.old_path == m_last_site.SitePath() && newSite && m_last_site.SameResource(*newSite)) {
+					if (m_last_site != *newSite) {
+						changed = true;
+						m_last_site.Update(*newSite);
+					}
+				}
+				else {
+					changed = true;
+					m_last_site.SetSitePath(std::wstring());
+				}
+			}
+			break;
+		}
+	}
+
+	if (changed) {
+		UpdateTitle();
+		NotifyHandlers(STATECHANGE_SERVER);
+	}
+}
+
 void CState::UpdateTitle()
 {
-	if (m_site.server_) {
-		wxString const& name = m_site.server_.server.GetName();
+	if (m_site) {
+		std::wstring const& name = m_site.server.GetName();
 		m_title.clear();
 		if (!name.empty()) {
 			m_title = name + _T(" - ");
 		}
-		m_title += m_site.server_.Format(ServerFormat::with_user_and_optional_port);
+		m_title += m_site.Format(ServerFormat::with_user_and_optional_port);
 	}
 	else {
 		m_title = _("Not connected");
