@@ -386,13 +386,13 @@ void CStatusBar::DoDisplayQueueSize()
 
 void CStatusBar::DisplayDataType()
 {
-	ServerWithCredentials server;
+	Site site;
 	CState const* pState = CContextManager::Get()->GetCurrentContext();
 	if (pState) {
-		server = pState->GetServer();
+		site = pState->GetSite();
 	}
 
-	if (!server || !CServer::ProtocolHasFeature(server.server.GetProtocol(), ProtocolFeature::DataTypeConcept)) {
+	if (!site || !CServer::ProtocolHasFeature(site.server.GetProtocol(), ProtocolFeature::DataTypeConcept)) {
 		if (m_pDataTypeIndicator) {
 			RemoveField(widget_datatype);
 			m_pDataTypeIndicator->Destroy();
@@ -450,16 +450,16 @@ void CStatusBar::MeasureQueueSizeWidth()
 
 void CStatusBar::DisplayEncrypted()
 {
-	ServerWithCredentials server;
+	Site site;
 	CState *const pState = CContextManager::Get()->GetCurrentContext();
 	if (pState) {
-		server = pState->GetServer();
+		site = pState->GetSite();
 	}
 
 	bool encrypted = false;
-	if (server) {
+	if (site) {
 		CCertificateNotification* info;
-		auto const protocol = server.server.GetProtocol();
+		auto const protocol = site.server.GetProtocol();
 		if (protocol == FTPS || protocol == FTPES || protocol == SFTP || protocol == S3||
 				protocol == WEBDAV || protocol == AZURE_BLOB || protocol == AZURE_FILE ||
 				protocol == SWIFT || protocol == GOOGLE_CLOUD || protocol == GOOGLE_DRIVE) {
@@ -561,7 +561,7 @@ void CStatusBar::ShowDataTypeMenu()
 {
 	wxMenu menu;
 	menu.Append(XRCID("ID_MENU_TRANSFER_TYPE_AUTO"), _("&Auto"), wxString(), wxITEM_RADIO);
-    menu.Append(XRCID("ID_MENU_TRANSFER_TYPE_ASCII"), _("A&SCII"), wxString(), wxITEM_RADIO); 
+	menu.Append(XRCID("ID_MENU_TRANSFER_TYPE_ASCII"), _("A&SCII"), wxString(), wxITEM_RADIO);
 	menu.Append(XRCID("ID_MENU_TRANSFER_TYPE_BINARY"), _("&Binary"), wxString(), wxITEM_RADIO);
 
 	const int type = COptions::Get()->GetOptionVal(OPTION_ASCIIBINARY);
@@ -649,7 +649,7 @@ void CStatusBar::OnOptionsChanged(changed_options_t const& options)
 	}
 }
 
-void CStatusBar::OnStateChange(CState*, t_statechange_notifications notification, const wxString&, const void*)
+void CStatusBar::OnStateChange(CState*, t_statechange_notifications notification, std::wstring const&, const void*)
 {
 	if (notification == STATECHANGE_SERVER || notification == STATECHANGE_CHANGEDCONTEXT) {
 		DisplayDataType();
