@@ -1,5 +1,5 @@
-#ifndef __STATUSBAR_H__
-#define __STATUSBAR_H__
+#ifndef FILEZILLA_INTERFACE_STATUSBAR_HEADER
+#define FILEZILLA_INTERFACE_STATUSBAR_HEADER
 
 #include <option_change_event_handler.h>
 #include "sizeformatting.h"
@@ -79,7 +79,7 @@ protected:
 		wxWindow* pChild;
 	};
 
-	std::map<int, struct t_statbar_child> m_children;
+	std::map<int, t_statbar_child> m_children;
 
 	void PositionChildren(int field);
 
@@ -87,7 +87,7 @@ protected:
 	void OnSize(wxSizeEvent& event);
 };
 
-class CStatusBar : public CWidgetsStatusBar, protected COptionChangeEventHandler, protected CGlobalStateEventHandler
+class CStatusBar final : public CWidgetsStatusBar, protected COptionChangeEventHandler, protected CGlobalStateEventHandler
 {
 public:
 	CStatusBar(wxTopLevelWindow* parent);
@@ -109,7 +109,7 @@ protected:
 	void ShowDataTypeMenu();
 
 	virtual void OnOptionsChanged(changed_options_t const& options);
-	virtual void OnStateChange(CState* pState, t_statechange_notifications notification, const wxString& data, const void* data2);
+	virtual void OnStateChange(CState* pState, t_statechange_notifications notification, std::wstring const& data, const void* data2) override;
 
 	void DoDisplayQueueSize();
 
@@ -132,4 +132,4 @@ protected:
 	void OnTimer(wxTimerEvent& ev);
 };
 
-#endif //__STATUSBAR_H__
+#endif
