@@ -289,7 +289,7 @@ bool CXmlFile::SaveXmlFile()
 	return true;
 }
 
-bool GetServer(pugi::xml_node node, ServerWithCredentials & server)
+bool GetServer(pugi::xml_node node, Site & site)
 {
 	wxASSERT(node);
 
@@ -303,7 +303,7 @@ bool GetServer(pugi::xml_node node, ServerWithCredentials & server)
 		return false;
 	}
 
-	if (!server.server.SetHost(host, port)) {
+	if (!site.server.SetHost(host, port)) {
 		return false;
 	}
 
@@ -311,30 +311,30 @@ bool GetServer(pugi::xml_node node, ServerWithCredentials & server)
 	if (protocol < 0 || protocol > ServerProtocol::MAX_VALUE) {
 		return false;
 	}
-	server.server.SetProtocol(static_cast<ServerProtocol>(protocol));
+	site.server.SetProtocol(static_cast<ServerProtocol>(protocol));
 
 	int type = GetTextElementInt(node, "Type");
 	if (type < 0 || type >= SERVERTYPE_MAX) {
 		return false;
 	}
 
-	server.server.SetType(static_cast<ServerType>(type));
+	site.server.SetType(static_cast<ServerType>(type));
 
 	int logonType = GetTextElementInt(node, "Logontype");
 	if (logonType < 0 || logonType >= static_cast<int>(LogonType::count)) {
 		return false;
 	}
 
-	server.SetLogonType(static_cast<LogonType>(logonType));
+	site.SetLogonType(static_cast<LogonType>(logonType));
 	
-	if (server.credentials.logonType_ != LogonType::anonymous) {
+	if (site.credentials.logonType_ != LogonType::anonymous) {
 		std::wstring user = GetTextElement(node, "User");
-		if (user.empty() && server.credentials.logonType_ != LogonType::interactive && server.credentials.logonType_ != LogonType::ask) {
+		if (user.empty() && site.credentials.logonType_ != LogonType::interactive && site.credentials.logonType_ != LogonType::ask) {
 			return false;
 		}
 
 		std::wstring pass, key;
-		if (server.credentials.logonType_ == LogonType::normal || server.credentials.logonType_ == LogonType::account) {
+		if (site.credentials.logonType_ == LogonType::normal || site.credentials.logonType_ == LogonType::account) {
 			auto passElement = node.child("Pass");
 			if (passElement) {
 				std::wstring encoding = GetTextAttribute(passElement, "encoding");
@@ -345,75 +345,75 @@ bool GetServer(pugi::xml_node node, ServerWithCredentials & server)
 				}
 				else if (encoding == _T("crypt")) {
 					pass = fz::to_wstring_from_utf8(passElement.child_value());
-					server.credentials.encrypted_ = fz::public_key::from_base64(passElement.attribute("pubkey").value());
-					if (!server.credentials.encrypted_) {
+					site.credentials.encrypted_ = fz::public_key::from_base64(passElement.attribute("pubkey").value());
+					if (!site.credentials.encrypted_) {
 						pass.clear();
-						server.SetLogonType(LogonType::ask);
+						site.SetLogonType(LogonType::ask);
 					}
 				}
 				else if (!encoding.empty()) {
-					server.SetLogonType(LogonType::ask);
+					site.SetLogonType(LogonType::ask);
 				}
 				else {
 					pass = GetTextElement(passElement);
 				}
 			}
 		}
-		else if (server.credentials.logonType_ == LogonType::key) {
+		else if (site.credentials.logonType_ == LogonType::key) {
 			key = GetTextElement(node, "Keyfile");
 
 			// password should be empty if we're using a key file
 			pass.clear();
 
-			server.credentials.keyFile_ = key;
+			site.credentials.keyFile_ = key;
 		}
 
-		server.SetUser(user);
-		server.credentials.SetPass(pass);
+		site.SetUser(user);
+		site.credentials.SetPass(pass);
 
-		server.credentials.account_ = GetTextElement(node, "Account");
+		site.credentials.account_ = GetTextElement(node, "Account");
 	}
 
 	int timezoneOffset = GetTextElementInt(node, "TimezoneOffset");
-	if (!server.server.SetTimezoneOffset(timezoneOffset)) {
+	if (!site.server.SetTimezoneOffset(timezoneOffset)) {
 		return false;
 	}
 
 	wxString pasvMode = GetTextElement(node, "PasvMode");
 	if (pasvMode == _T("MODE_PASSIVE")) {
-		server.server.SetPasvMode(MODE_PASSIVE);
+		site.server.SetPasvMode(MODE_PASSIVE);
 	}
 	else if (pasvMode == _T("MODE_ACTIVE")) {
-		server.server.SetPasvMode(MODE_ACTIVE);
+		site.server.SetPasvMode(MODE_ACTIVE);
 	}
 	else {
-		server.server.SetPasvMode(MODE_DEFAULT);
+		site.server.SetPasvMode(MODE_DEFAULT);
 	}
 
 	int maximumMultipleConnections = GetTextElementInt(node, "MaximumMultipleConnections");
-	server.server.MaximumMultipleConnections(maximumMultipleConnections);
+	site.server.MaximumMultipleConnections(maximumMultipleConnections);
 
 	wxString encodingType = GetTextElement(node, "EncodingType");
 	if (encodingType == _T("Auto")) {
-		server.server.SetEncodingType(ENCODING_AUTO);
+		site.server.SetEncodingType(ENCODING_AUTO);
 	}
 	else if (encodingType == _T("UTF-8")) {
-		server.server.SetEncodingType(ENCODING_UTF8);
+		site.server.SetEncodingType(ENCODING_UTF8);
 	}
 	else if (encodingType == _T("Custom")) {
 		std::wstring customEncoding = GetTextElement(node, "CustomEncoding");
 		if (customEncoding.empty()) {
 			return false;
 		}
-		if (!server.server.SetEncodingType(ENCODING_CUSTOM, customEncoding)) {
+		if (!site.server.SetEncodingType(ENCODING_CUSTOM, customEncoding)) {
 			return false;
 		}
 	}
 	else {
-		server.server.SetEncodingType(ENCODING_AUTO);
+		site.server.SetEncodingType(ENCODING_AUTO);
 	}
 
-	if (CServer::ProtocolHasFeature(server.server.GetProtocol(), ProtocolFeature::PostLoginCommands)) {
+	if (CServer::ProtocolHasFeature(site.server.GetProtocol(), ProtocolFeature::PostLoginCommands)) {
 		std::vector<std::wstring> postLoginCommands;
 		auto element = node.child("PostLoginCommands");
 		if (element) {
@@ -424,26 +424,26 @@ bool GetServer(pugi::xml_node node, ServerWithCredentials & server)
 				}
 			}
 		}
-		if (!server.server.SetPostLoginCommands(postLoginCommands)) {
+		if (!site.server.SetPostLoginCommands(postLoginCommands)) {
 			return false;
 		}
 	}
 
-	server.server.SetBypassProxy(GetTextElementInt(node, "BypassProxy", false) == 1);
-	server.server.SetName(GetTextElement_Trimmed(node, "Name"));
+	site.server.SetBypassProxy(GetTextElementInt(node, "BypassProxy", false) == 1);
+	site.server.SetName(GetTextElement_Trimmed(node, "Name"));
 
-	if (server.server.GetName().empty()) {
-		server.server.SetName(GetTextElement_Trimmed(node));
+	if (site.server.GetName().empty()) {
+		site.server.SetName(GetTextElement_Trimmed(node));
 	}
 
 	for (auto parameter = node.child("Parameter"); parameter; parameter = parameter.next_sibling("Parameter")) {
-		server.server.SetExtraParameter(parameter.attribute("Name").value(), GetTextElement(parameter));
+		site.server.SetExtraParameter(parameter.attribute("Name").value(), GetTextElement(parameter));
 	}
 
 	return true;
 }
 
-void SetServer(pugi::xml_node node, ServerWithCredentials const& server)
+void SetServer(pugi::xml_node node, Site const& site)
 {
 	if (!node) {
 		return;
@@ -453,17 +453,17 @@ void SetServer(pugi::xml_node node, ServerWithCredentials const& server)
 		node.remove_child(child);
 	}
 
-	ServerProtocol const protocol = server.server.GetProtocol();
+	ServerProtocol const protocol = site.server.GetProtocol();
 
-	AddTextElement(node, "Host", server.server.GetHost());
-	AddTextElement(node, "Port", server.server.GetPort());
+	AddTextElement(node, "Host", site.server.GetHost());
+	AddTextElement(node, "Port", site.server.GetPort());
 	AddTextElement(node, "Protocol", protocol);
-	AddTextElement(node, "Type", server.server.GetType());
+	AddTextElement(node, "Type", site.server.GetType());
 
-	ProtectedCredentials credentials = server.credentials;
+	ProtectedCredentials credentials = site.credentials;
 
 	if (credentials.logonType_ != LogonType::anonymous) {
-		AddTextElement(node, "User", server.server.GetUser());
+		AddTextElement(node, "User", site.server.GetUser());
 
 		credentials.Protect();
 
@@ -494,8 +494,8 @@ void SetServer(pugi::xml_node node, ServerWithCredentials const& server)
 	}
 	AddTextElement(node, "Logontype", static_cast<int>(credentials.logonType_));
 
-	AddTextElement(node, "TimezoneOffset", server.server.GetTimezoneOffset());
-	switch (server.server.GetPasvMode())
+	AddTextElement(node, "TimezoneOffset", site.server.GetTimezoneOffset());
+	switch (site.server.GetPasvMode())
 	{
 	case MODE_PASSIVE:
 		AddTextElementUtf8(node, "PasvMode", "MODE_PASSIVE");
@@ -507,9 +507,9 @@ void SetServer(pugi::xml_node node, ServerWithCredentials const& server)
 		AddTextElementUtf8(node, "PasvMode", "MODE_DEFAULT");
 		break;
 	}
-	AddTextElement(node, "MaximumMultipleConnections", server.server.MaximumMultipleConnections());
+	AddTextElement(node, "MaximumMultipleConnections", site.server.MaximumMultipleConnections());
 
-	switch (server.server.GetEncodingType())
+	switch (site.server.GetEncodingType())
 	{
 	case ENCODING_AUTO:
 		AddTextElementUtf8(node, "EncodingType", "Auto");
@@ -519,12 +519,12 @@ void SetServer(pugi::xml_node node, ServerWithCredentials const& server)
 		break;
 	case ENCODING_CUSTOM:
 		AddTextElementUtf8(node, "EncodingType", "Custom");
-		AddTextElement(node, "CustomEncoding", server.server.GetCustomEncoding());
+		AddTextElement(node, "CustomEncoding", site.server.GetCustomEncoding());
 		break;
 	}
 
-	if (CServer::ProtocolHasFeature(server.server.GetProtocol(), ProtocolFeature::PostLoginCommands)) {
-		std::vector<std::wstring> const& postLoginCommands = server.server.GetPostLoginCommands();
+	if (CServer::ProtocolHasFeature(site.server.GetProtocol(), ProtocolFeature::PostLoginCommands)) {
+		std::vector<std::wstring> const& postLoginCommands = site.server.GetPostLoginCommands();
 		if (!postLoginCommands.empty()) {
 			auto element = node.append_child("PostLoginCommands");
 			for (auto const& command : postLoginCommands) {
@@ -533,13 +533,13 @@ void SetServer(pugi::xml_node node, ServerWithCredentials const& server)
 		}
 	}
 
-	AddTextElementUtf8(node, "BypassProxy", server.server.GetBypassProxy() ? "1" : "0");
-	std::wstring const& name = server.server.GetName();
+	AddTextElementUtf8(node, "BypassProxy", site.server.GetBypassProxy() ? "1" : "0");
+	std::wstring const& name = site.server.GetName();
 	if (!name.empty()) {
 		AddTextElement(node, "Name", name);
 	}
 
-	for (auto const& parameter : server.server.GetExtraParameters()) {
+	for (auto const& parameter : site.server.GetExtraParameters()) {
 		auto element = AddTextElement(node, "Parameter", parameter.second);
 		SetTextAttribute(element, "Name", parameter.first);
 	}
